//////////////////////////////////////////////////////////////////////////////
//
//    BORDERIMAGESDIALOG.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// C Ansi includes

extern "C"
{
#include <unistd.h>
}

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qprocess.h>
#include <qpainter.h>
#include <qpalette.h>

// Include files for KDE

#include <kstandarddirs.h>
#include <klocale.h>
#include <kprogress.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kcolorbutton.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kimageio.h>
#include <kprocess.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/digikamio.h>

// Local includes

#include "borderimagesdialog.h"
#include "borderoptionsdialog.h"
#include "outputdialog.h"
#include "imagepreview.h"


class ImageFileBorderItem : public KListViewItem
{
public:
    ImageFileBorderItem(QListView * parent, QString const & nameSrc, QString const & result)
      : KListViewItem( parent, nameSrc, result), _nameSrc(nameSrc), _result(result)
    {}

    QString nameSrc()                   { return _nameSrc; }
    QString result()                    { return _result; }
    QString error()                     { return _error; }
    QString outputMess()                { return _outputMess; }

    void changeResult(QString text)     { setText(1, text); }
    void changeError(QString text)      { _error = text; }
    void changeOutputMess(QString text) { _outputMess.append(text); }

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(1) != i18n("OK") && text(1) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(1) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString _nameSrc;
    QString _result;
    QString _error;
    QString _outputMess;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

BorderImagesDialog::BorderImagesDialog(QWidget *parent, QStringList filesList)
                 : KDialogBase( parent, "BorderImagesDialog", true,
                 i18n("Batch Bordering Images options"),
                 Help|User1|User2|Cancel, Cancel, true, i18n("&About"), i18n("&Start"))
{
    // Init. Tmp folder

    KStandardDirs dir;
    m_tmpFolder = dir.saveLocation("tmp");

    m_processStatus = NO_PROCESS;
    m_progressStatus = 0;
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_ProcessusProc = 0L;
    m_PreviewProc = 0L;

    setHelp("plugin-borderimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Border images options"), box );

    m_labelBorderType = new QLabel( i18n("Border type:"), groupBox1);

    m_borderType = new QComboBox(false, groupBox1);
    m_borderType->insertItem(i18n("Solid"));
    m_borderType->insertItem(i18n("Niepce"));
    m_borderType->insertItem(i18n("Raise"));
    m_borderType->insertItem(i18n("Frame"));
    m_borderType->setCurrentText(i18n("Niepce"));
    whatsThis = i18n("<p>Select here the border type for your images:<p>"
                     "<b>Solid</b>: just surround the images with a line.<p>"
                     "<b>Niepce</b>: surround the images with a fine line and a large border "
                     "(ideal for black and white pictures).<p>"
                     "<b>Raise</b>: creating a 3D effect arround the images.<p>"
                     "<b>Frame</b>: surround the images with an ornamental border.<p>");
    QWhatsThis::add( m_borderType, whatsThis );
    m_labelBorderType->setBuddy( m_borderType );

    m_optionsButton = new QPushButton (groupBox1, "OptionButton");
    m_optionsButton->setText(i18n("&Options"));
    QWhatsThis::add( m_optionsButton, i18n("<p>You can choose here the current border type "
                                           "options using for the process."));

    m_previewButton = new QPushButton (groupBox1, "PreviewButton");
    m_previewButton->setText(i18n("&Preview"));
    QWhatsThis::add( m_previewButton, i18n("<p>You can build with this button an process "
                                           "preview for the current selected image in the list."));
    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 1, Qt::Horizontal,
                                i18n("Bordering images (%1 items)").arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox3 );
    m_listFiles->addColumn(i18n("Source image"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(1);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 250 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here the operations results "
                                       "during the bordering. Double click on one item for more "
                                       "informations after the process.") );

    m_statusbar = new QLabel( groupBox3, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>This is the current active task.") );

    m_progress = new KProgress( groupBox3, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    connect(m_listFiles, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotListDoubleClicked(QListViewItem *)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));

    connect(m_optionsButton, SIGNAL(clicked()),
            this, SLOT(slotOptionsClicked()));

    connect(m_previewButton, SIGNAL(clicked()),
            this, SLOT(slotPreview()));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("BorderImages Settings");

    m_borderType->setCurrentText(m_config->readEntry("BorderType", "Niepce"));
    QColor *ColorBlack = new QColor( 0, 0, 0 );
    QColor *ColorWhite = new QColor( 255, 255, 255 );
    m_solidWidth = m_config->readNumEntry("SolidWidth", 25);
    m_solidColor = m_config->readColorEntry("SolidColor", ColorBlack);

    m_lineNiepceWidth = m_config->readNumEntry("LineNiepceWidth", 10);
    m_lineNiepceColor = m_config->readColorEntry("LineNiepceColor", ColorBlack);
    m_NiepceWidth = m_config->readNumEntry("NiepceWidth", 100);
    m_NiepceColor = m_config->readColorEntry("NiepceColor", ColorWhite);

    m_raiseWidth = m_config->readNumEntry("RaiseWidth", 50);

    m_frameWidth = m_config->readNumEntry("FrameWidth", 25);
    m_bevelWidth = m_config->readNumEntry("BevelWidth", 10);
    m_frameColor = m_config->readColorEntry("FrameColor", ColorBlack);

    listImageFiles();
    delete m_config;
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

BorderImagesDialog::~BorderImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////

void BorderImagesDialog::closeEvent ( QCloseEvent *e )
{
    if (!e) return;

    if ( m_PreviewProc != 0L )
       if ( m_PreviewProc->isRunning() ) m_PreviewProc->kill(SIGTERM);

    if ( m_ProcessusProc != 0L )
       if ( m_ProcessusProc->isRunning() ) m_ProcessusProc->kill(SIGTERM);

    e->accept();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch bordering images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n\n"
                                  "This plugin use the \"mogrify\" program from \"ImageMagick\" package.\n"),
                                  i18n("About Digikam batch bordering images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotOptionsClicked(void)
{
    QString Type = m_borderType->currentText();
    BorderOptionsDialog *optionsDialog = new BorderOptionsDialog(0, Type);

    if ( Type == i18n("Solid") )
       {
       optionsDialog->m_solidBorderWidth->setValue(m_solidWidth);
       optionsDialog->m_button_solidBorderColor->setColor(m_solidColor);
       }
    if ( Type == i18n("Niepce") )
       {
       optionsDialog->m_lineNiepceBorderWidth->setValue(m_lineNiepceWidth);
       optionsDialog->m_button_lineNiepceBorderColor->setColor(m_lineNiepceColor);
       optionsDialog->m_NiepceBorderWidth->setValue(m_NiepceWidth);
       optionsDialog->m_button_NiepceBorderColor->setColor(m_NiepceColor);
       }
    if ( Type == i18n("Raise") )
       {
       optionsDialog->m_raiseBorderWidth->setValue(m_raiseWidth);
       }
    if ( Type == i18n("Frame") )
       {
       optionsDialog->m_frameBorderWidth->setValue(m_frameWidth);
       optionsDialog->m_frameBevelBorderWidth->setValue(m_bevelWidth);
       optionsDialog->m_button_frameBorderColor->setColor(m_frameColor);
       }

    if ( optionsDialog->exec() == KMessageBox::Ok )
       {
       if ( Type == i18n("Solid") )
          {
          m_solidWidth = optionsDialog->m_solidBorderWidth->value();
          m_solidColor = optionsDialog->m_button_solidBorderColor->color();
          }
       if ( Type == i18n("Niepce") )
          {
          m_lineNiepceWidth = optionsDialog->m_lineNiepceBorderWidth->value();
          m_lineNiepceColor = optionsDialog->m_button_lineNiepceBorderColor->color();
          m_NiepceWidth = optionsDialog->m_NiepceBorderWidth->value();
          m_NiepceColor = optionsDialog->m_button_NiepceBorderColor->color();
          }
       if ( Type == i18n("Raise") )
          {
          m_raiseWidth = optionsDialog->m_raiseBorderWidth->value();
          }
       if ( Type == i18n("Frame") )
          {
          m_frameWidth = optionsDialog->m_frameBorderWidth->value();
          m_bevelWidth = optionsDialog->m_frameBevelBorderWidth->value();
          m_frameColor = optionsDialog->m_button_frameBorderColor->color();
          }
       }

   delete optionsDialog;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotProcessStart( void )
{
    m_processStatus = UNDER_PROCESS;
    m_statusbar->setText(i18n("Process under progress. Please wait..."));

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStop()));

    m_labelBorderType->setEnabled(false);
    m_borderType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_previewButton->setEnabled(false);

    m_listFile2Process_iterator = new QListViewItemIterator( m_listFiles );
    startProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool BorderImagesDialog::startProcess(void)
{
    if ( m_processStatus == STOP_PROCESS )
       {
       endProcess();
       return true;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->setCurrentItem(item);

    m_commandLine ="mogrify";
    m_ProcessusProc = new KProcess;
    *m_ProcessusProc << "mogrify";

    m_commandLine.append(makeProcess(m_ProcessusProc, album, item));

    *m_ProcessusProc << "-verbose";
    *m_ProcessusProc << album->getPath() + "/" + item->nameSrc();
    m_commandLine.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + "\n\n");

    item->changeOutputMess(m_commandLine);

    connect(m_ProcessusProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(ProcessDone(KProcess*)));

    connect(m_ProcessusProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));
            
    connect(m_ProcessusProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    bool result = m_ProcessusProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'mogrify' program from 'ImageMagick' package.\n"
                                     "Please, check your installation!"));
       return false;
       }

    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString BorderImagesDialog::makeProcess(KProcess* proc, Digikam::AlbumInfo *album,
                                        ImageFileBorderItem *item)
{
    QString outputMess;
    KURL desturl(album->getPath() + "/" + item->nameSrc());

    if (m_borderType->currentText() == i18n("Solid"))
       {
       *proc << "-border";
       outputMess.append( " -border " );
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_solidWidth ) + "x";
       Temp2.append(Temp.setNum( m_solidWidth ));
       *proc << Temp2;
       outputMess.append( Temp2 );
       *proc << "-bordercolor";
       outputMess.append( " -bordercolor " );
       Temp2 = "rgb(" + Temp.setNum(m_solidColor.red()) + ",";
       Temp2.append(Temp.setNum(m_solidColor.green()) + ",");
       Temp2.append(Temp.setNum(m_solidColor.blue()) + ")");
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    if (m_borderType->currentText() == i18n("Niepce"))
       {
       QString Temp, Temp2;

       *proc << "-border";
       outputMess.append( " -border " );
       Temp2 = Temp.setNum( m_lineNiepceWidth ) + "x";
       Temp2.append(Temp.setNum( m_lineNiepceWidth ));
       *proc << Temp2;
       outputMess.append( Temp2 );

       *proc << "-bordercolor";
       outputMess.append( " -bordercolor " );
       Temp2 = "rgb(" + Temp.setNum(m_NiepceColor.red()) + ",";
       Temp2.append(Temp.setNum(m_NiepceColor.green()) + ",");
       Temp2.append(Temp.setNum(m_NiepceColor.blue()) + ")");
       *proc << Temp2;
       outputMess.append( Temp2 );

       *proc << "-border";
       outputMess.append( " -border " );
       Temp2 = Temp.setNum( m_NiepceWidth ) + "x";
       Temp2.append(Temp.setNum( m_NiepceWidth ));
       *proc << Temp2;
       outputMess.append( Temp2 );

       *proc << "-bordercolor";
       outputMess.append( " -bordercolor " );
       Temp2 = "rgb(" + Temp.setNum(m_lineNiepceColor.red()) + ",";
       Temp2.append(Temp.setNum(m_lineNiepceColor.green()) + ",");
       Temp2.append(Temp.setNum(m_lineNiepceColor.blue()) + ")");
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    if (m_borderType->currentText() == i18n("Raise"))
       {
       *proc << "-raise";
       outputMess.append( " -raise " );
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_raiseWidth ) + "x";
       Temp2.append(Temp.setNum( m_raiseWidth ));
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    if (m_borderType->currentText() == i18n("Frame"))
       {
       *proc << "-frame";
       outputMess.append( " -frame " );
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_frameWidth ) + "x";
       Temp2.append(Temp.setNum( m_frameWidth ) + "+");
       Temp2.append(Temp.setNum( m_bevelWidth ) + "+");
       Temp2.append(Temp.setNum( m_bevelWidth ) );
       *proc << Temp2;
       outputMess.append( Temp2 );
       *proc << "-mattecolor";
       outputMess.append( " -mattecolor " );
       Temp2 = "rgb(" + Temp.setNum(m_frameColor.red()) + ",";
       Temp2.append(Temp.setNum(m_frameColor.green()) + ",");
       Temp2.append(Temp.setNum(m_frameColor.blue()) + ")");
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    return(outputMess);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotReadStd(KProcess* proc, char *buffer, int buflen)
{
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( m_listFile2Process_iterator->current() );
    item->changeOutputMess( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::ProcessDone(KProcess* proc)
{
    int ValRet = proc->exitStatus();
    qDebug ("Mogrify exit (%i)", ValRet);

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());

    switch (ValRet)
       {
       case 0:       // Process finished successfully !
          {
          item->changeResult(i18n("OK"));
          item->changeError(i18n("no processing error"));
          break;
          }
       case 15:      //  process aborted !
          {
          processAborted();
          break;
          }
       default :     // Processing error !
          {
          item->changeResult(i18n("Failed !!!"));
          item->changeError(i18n("cannot processing image file!"));
          break;
          }
       }

    ++*m_listFile2Process_iterator;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if ( m_listFile2Process_iterator->current() )
       startProcess();
    else
       {
       m_statusbar->setText(i18n("Process finished!"));
       endProcess();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("BorderImages Settings");
    m_config->writeEntry("BorderType", m_borderType->currentText());

    m_config->writeEntry("SolidWidth", m_solidWidth);
    m_config->writeEntry("SolidColor", m_solidColor);

    m_config->writeEntry("LineNiepceWidth", m_lineNiepceWidth);
    m_config->writeEntry("LineNiepceColor", m_lineNiepceColor);
    m_config->writeEntry("NiepceWidth", m_NiepceWidth);
    m_config->writeEntry("NiepceColor", m_NiepceColor);

    m_config->writeEntry("RaiseWidth", m_raiseWidth);

    m_config->writeEntry("FrameWidth", m_frameWidth);
    m_config->writeEntry("BevelWidth", m_bevelWidth);
    m_config->writeEntry("FrameColor", m_frameColor);
    m_config->sync();

    delete m_config;
    delete this;
}


////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void BorderImagesDialog::listImageFiles(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, QDir::Name);
    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileBorderItem *item = new ImageFileBorderItem(m_listFiles,
                                                              fi->fileName(),
                                                              ""
                                                              );
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotListDoubleClicked(QListViewItem *itemClicked)
{
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( itemClicked );

    if (m_processStatus == PROCESS_DONE)
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Image error processing"),
                                                   item->outputMess(),
                                                   i18n("Image \"%1\": %2\n\nThe ouput messages are:\n").arg(item->nameSrc()).arg(item->error())
                                                   );
       infoDialog->exec();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotProcessStop( void )
{
    // Try to kill the current process !
    if ( m_ProcessusProc->isRunning() == true ) m_ProcessusProc->kill(SIGTERM);

    // If kill operation failed, Stop the process at the next image !
    if ( m_processStatus == UNDER_PROCESS ) m_processStatus = STOP_PROCESS;

    processAborted();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotPreview(void)
{
    if ( m_listFiles->currentItem() == 0 )
       {
       KMessageBox::error(this, i18n("You must selected an item in the list for calculate the preview!"));
       return;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( m_listFiles->currentItem() );
    m_statusbar->setText(i18n("Preview under progress. Please wait..."));

    m_listFiles->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_labelBorderType->setEnabled(false);
    m_borderType->setEnabled(false);
    m_optionsButton->setEnabled(false);

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotPreviewStop()));

    m_previewOutput ="convert";
    m_PreviewProc = new KProcess;
    *m_PreviewProc << "convert";

    m_previewOutput.append(makeProcess(m_PreviewProc, album, item));

    *m_PreviewProc << "-verbose";
    *m_PreviewProc << album->getPath() + "/" + item->nameSrc();
    *m_PreviewProc << m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG";
    m_previewOutput.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + " "
                            + m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG\n\n");

    connect(m_PreviewProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(PreviewProcessDone(KProcess*)));

    connect(m_PreviewProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    connect(m_PreviewProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    bool result = m_PreviewProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'convert' program from 'ImageMagick' package.\n"
                                     "Please, check your installation!"));
       m_previewButton->setEnabled(true);
       return;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotPreviewReadStd(KProcess* proc, char *buffer, int buflen)
{
    m_previewOutput.append( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::PreviewProcessDone(KProcess* proc)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( m_listFiles->currentItem() );
    int ValRet = proc->exitStatus();
    qDebug ("Convert exit (%i)", ValRet);

    if ( ValRet == 0 )
       {
       ImagePreview *previewDialog = new ImagePreview(album->getPath() + "/" + item->nameSrc(),
                                         m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG",
                                         m_tmpFolder, false, false, m_borderType->currentText());
       previewDialog->exec();

       KURL deletePreviewImage(m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG");
       KIO::NetAccess::del(deletePreviewImage);
       }
    else
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Preview error processing"),
                                                   m_previewOutput,
                                                   i18n("Cannot process preview for image \"%1\"."
                                                        "\nThe ouput messages are:\n")
                                                        .arg(item->nameSrc())
                                                   );
       infoDialog->exec();
       }

    endPreview();
    m_statusbar->setText("");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::slotPreviewStop( void )
{
    // Try to kill the current preview process !
    if ( m_PreviewProc->isRunning() == true ) m_PreviewProc->kill(SIGTERM);

    endPreview();
    m_statusbar->setText("Preview aborted by user!");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::endPreview(void)
{
    m_listFiles->setEnabled(true);
    m_previewButton->setEnabled(true);
    m_labelBorderType->setEnabled(true);
    m_optionsButton->setEnabled(true);
    m_borderType->setEnabled(true);
    showButtonCancel( true );

    setButtonText( User2, i18n("&Start") );
    disconnect(this, SIGNAL(user2Clicked()), this, SLOT(slotPreviewStop()));
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::endProcess(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumManager::instance()->refreshItemHandler(album->getTitle());
    m_processStatus = PROCESS_DONE;
    setButtonText( User2, i18n("&Close") );

    disconnect(this, SIGNAL(user2Clicked()),
               this, SLOT(slotProcessStop()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotOk()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void BorderImagesDialog::processAborted(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileBorderItem *item = static_cast<ImageFileBorderItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());
    m_statusbar->setText(i18n("Process aborted by user!"));

    item->changeResult(i18n("Aborted !"));
    item->changeError(i18n("process aborted by user"));
    endProcess();
}
