//////////////////////////////////////////////////////////////////////////////
//
//    ColorImagesDialog.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// C Ansi includes

extern "C"
{
#include <unistd.h>
}

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qprocess.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpalette.h>

// Include files for KDE

#include <kstandarddirs.h>
#include <klocale.h>
#include <kprogress.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kimageio.h>
#include <kprocess.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/digikamio.h>

// Local includes

#include "colorimagesdialog.h"
#include "coloroptionsdialog.h"
#include "outputdialog.h"
#include "imagepreview.h"


class ImageFileColorItem : public KListViewItem
{
public:
    ImageFileColorItem(QListView * parent, QString const & nameSrc, QString const & result)
      : KListViewItem( parent, nameSrc, result), _nameSrc(nameSrc), _result(result)
    {}

    QString nameSrc()                   { return _nameSrc; }
    QString result()                    { return _result; }
    QString error()                     { return _error; }
    QString outputMess()                { return _outputMess; }

    void changeResult(QString text)     { setText(1, text); }
    void changeError(QString text)      { _error = text; }
    void changeOutputMess(QString text) { _outputMess.append(text); }

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(1) != i18n("OK") && text(1) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(1) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString _nameSrc;
    QString _result;
    QString _error;
    QString _outputMess;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

ColorImagesDialog::ColorImagesDialog(QWidget *parent, QStringList filesList)
                 : KDialogBase( parent, "ColorImagesDialog", true,
                 i18n("Batch Coloring Images options"), Help|User1|User2|Cancel, Cancel,
                      true, i18n("&About"), i18n("&Start"))
{
    // Init. Tmp folder
    KStandardDirs dir;
    m_tmpFolder = dir.saveLocation("tmp");

    m_processStatus = NO_PROCESS;
    m_progressStatus = 0;
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_ProcessusProc = 0L;
    m_PreviewProc = 0L;

    setHelp("plugin-colorimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Color images options"), box );

    m_labelColorType = new QLabel( i18n("Colors enhancement type:"), groupBox1);

    m_colorType = new QComboBox(false, groupBox1);
    m_colorType->insertItem(i18n("Decrease contrast"));
    m_colorType->insertItem(i18n("Depth"));
    m_colorType->insertItem(i18n("Equalize"));
    m_colorType->insertItem(i18n("Fuzz"));
    m_colorType->insertItem(i18n("Gray scales"));
    m_colorType->insertItem(i18n("Increase contrast"));
    m_colorType->insertItem(i18n("Monochrome"));
    m_colorType->insertItem(i18n("Negate"));
    m_colorType->insertItem(i18n("Normalize"));
    m_colorType->insertItem(i18n("Segment"));
    m_colorType->insertItem(i18n("Trim"));
    m_colorType->setCurrentText(i18n("Normalize"));
    whatsThis = i18n("<p>Select here the colors enhancement type for your images:<p>"
                     "<b>Decrease contrast</b>: reduce the image contrast. The algorithm "
                     "reduce the intensity differences between the lighter and darker elements "
                     "of the image.<p>"
                     "<b>Depth</b>: change the color depth of the image.<p>"
                     "<b>Equalize</b>: perform histogram equalization to the image.<p>"
                     "<b>Fuzz</b>: merging colors within a distance are considered equal.<p>"
                     "<b>Gray scales</b>: convert colors image to gray scales image.<p>"
                     "<b>Increase contrast</b>: enhance the image contrast. The algorithm enhance "
                     "the intensity differences between the lighter and darker elements of the image.<p>"
                     "<b>Monochrome</b>: transform the image to black and white.<p>"
                     "<b>Negate</b>: replace every pixel with its complementary color. The red, green, "
                     "and blue intensities of an image are negated. White becomes black, yellow becomes "
                     "blue, etc.<p>"
                     "<b>Normalize</b>: transform image to span the full range of color values. "
                     "This is a contrast enhancement technique. The algorithm enhances the contrast "
                     "of a colored image by adjusting the pixels color to span the entire range of "
                     "colors available.<p>"
                     "<b>Segment</b>: segment an image by analyzing the histograms of the color "
                     "components and identifying units that are homogeneous with the fuzzy c-means "
                     "technique.<p>"
                     "<b>Trim</b>: trim an image (fuzz reverse technic). The algorithm remove edges "
                     "that are the background color from the image.<p>");

    QWhatsThis::add( m_colorType, whatsThis );
    m_labelColorType->setBuddy( m_colorType );

    m_optionsButton = new QPushButton (groupBox1, "OptionButton");
    m_optionsButton->setText(i18n("&Options"));
    QWhatsThis::add( m_optionsButton, i18n("<p>You can choose here the current border type options using "
                                           "for the process."));

    m_previewButton = new QPushButton (groupBox1, "PreviewButton");
    m_previewButton->setText(i18n("&Preview"));
    QWhatsThis::add( m_previewButton, i18n("<p>You can build with this button an process preview for "
                                           "the current selected image in the list."));

    m_smallPreview = new QCheckBox(i18n("Small preview rendering"), groupBox1);
    QWhatsThis::add( m_smallPreview, i18n("<p>If you enable this option, "
                                          "all preview effects will be calculed on a small zone "
                                          "of the image (300x300 pixels on the top left corner). "
                                          "Enabled this option if you have a slow computer.") );
    m_smallPreview->setChecked( true );

    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 1, Qt::Horizontal, i18n("Coloring images (%1 items)")
                                                              .arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox3 );
    m_listFiles->addColumn(i18n("Source image"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(1);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 250 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here the operations results during the filtering. "
                                       "Double click on one item for more informations after the process.") );

    m_statusbar = new QLabel( groupBox3, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>This is the current active task.") );

    m_progress = new KProgress( groupBox3, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    connect(m_listFiles, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotListDoubleClicked(QListViewItem *)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));

    connect(m_optionsButton, SIGNAL(clicked()),
            this, SLOT(slotOptionsClicked()));

    connect(m_previewButton, SIGNAL(clicked()),
            this, SLOT(slotPreview()));

    connect(m_colorType, SIGNAL(activated(const QString &)),
            this, SLOT(slotColorTypeChanged(const QString &)));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("ColorImages Settings");

    m_colorType->setCurrentText(m_config->readEntry("ColorType", i18n("Normalize")));
    m_depthValue = m_config->readEntry("DepthValue", "32");
    m_fuzzDistance = m_config->readNumEntry("FuzzDistance", 3);
    m_segmentCluster = m_config->readNumEntry("SegmentCluster", 3);
    m_segmentSmooth = m_config->readNumEntry("SegmentSmooth", 3);

    if (m_config->readEntry("SmallPreview", "true") == "true")
        m_smallPreview->setChecked( true );
    else
        m_smallPreview->setChecked( false );

    //---------------------------------------------

    listImageFiles();
    slotColorTypeChanged(m_colorType->currentText());
    delete m_config;
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

ColorImagesDialog::~ColorImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////

void ColorImagesDialog::closeEvent ( QCloseEvent *e )
{
    if (!e) return;

    if ( m_PreviewProc != 0L )
       if ( m_PreviewProc->isRunning() ) m_PreviewProc->kill(SIGTERM);

    if ( m_ProcessusProc != 0L )
       if ( m_ProcessusProc->isRunning() ) m_ProcessusProc->kill(SIGTERM);

    e->accept();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch color enhancing images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n\n"
                                  "This plugin use the \"mogrify\" program from \"ImageMagick\" package.\n"),
                                  i18n("About Digikam batch color enhancing images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotColorTypeChanged(const QString &string)
{
    if ( string == i18n("Depth") || string == i18n("Fuzz") || string == i18n("Segment") )
       m_optionsButton->setEnabled(true);
    else
       m_optionsButton->setEnabled(false);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotOptionsClicked(void)
{
    QString Type = m_colorType->currentText();
    ColorOptionsDialog *optionsDialog = new ColorOptionsDialog(0, Type);

    if ( Type == i18n("Depth") )
       optionsDialog->m_depthValue->setCurrentText(m_depthValue);

    if ( Type == i18n("Fuzz") )
       optionsDialog->m_fuzzDistance->setValue(m_fuzzDistance);

    if ( Type == i18n("Segment") )
       {
       optionsDialog->m_segmentCluster->setValue(m_segmentCluster);
       optionsDialog->m_segmentSmooth->setValue(m_segmentSmooth);
       }

    if ( optionsDialog->exec() == KMessageBox::Ok )
       {
       if ( Type == i18n("Depth") )
          m_depthValue = optionsDialog->m_depthValue->currentText();

       if ( Type == i18n("Fuzz") )
          m_fuzzDistance = optionsDialog->m_fuzzDistance->value();

       if ( Type == i18n("Segment") )
          {
          m_segmentCluster = optionsDialog->m_segmentCluster->value();
          m_segmentSmooth = optionsDialog->m_segmentSmooth->value();
          }
       }

    delete optionsDialog;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotProcessStart( void )
{
    m_processStatus = UNDER_PROCESS;
    m_statusbar->setText(i18n("Process under progress. Please wait..."));

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStop()));

    m_labelColorType->setEnabled(false);
    m_colorType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_smallPreview->setEnabled(false);

    m_listFile2Process_iterator = new QListViewItemIterator( m_listFiles );
    startProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool ColorImagesDialog::startProcess(void)
{
    if ( m_processStatus == STOP_PROCESS )
       {
       endProcess();
       return true;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->setCurrentItem(item);

    m_commandLine ="mogrify";
    m_ProcessusProc = new KProcess;
    *m_ProcessusProc << "mogrify";

    m_commandLine.append(makeProcess(m_ProcessusProc, album, item));

    *m_ProcessusProc << "-verbose";
    *m_ProcessusProc << album->getPath() + "/" + item->nameSrc();
    m_commandLine.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + "\n\n");

    item->changeOutputMess(m_commandLine);

    connect(m_ProcessusProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(ProcessDone(KProcess*)));

    connect(m_ProcessusProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    connect(m_ProcessusProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    bool result = m_ProcessusProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'mogrify' program from 'ImageMagick' package."
                                     "\nPlease, check your installation!"));
       return false;
       }

    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString ColorImagesDialog::makeProcess(KProcess* proc, Digikam::AlbumInfo *album,
                                       ImageFileColorItem *item)
{
    QString outputMess;
    KURL desturl(album->getPath() + "/" + item->nameSrc());

    if (m_colorType->currentText() == i18n("Decrease contrast"))
       {
       *proc << "-contrast";
       outputMess.append( " -contrast " );
       }

    if (m_colorType->currentText() == i18n("Depth"))
       {
       *proc << "-depth" << m_depthValue;
       outputMess.append( " -depth " + m_depthValue );
       }

    if (m_colorType->currentText() == i18n("Equalize"))
       {
       *proc << "-equalize";
       outputMess.append( " -equalize " );
       }

    if (m_colorType->currentText() == i18n("Fuzz"))
       {
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_fuzzDistance );
       *proc << "-fuzz" << Temp2;
       outputMess.append( " -fuzz " + Temp2 );
       }

    if (m_colorType->currentText() == i18n("Gray scales"))
       {
       *proc << "-type";
       *proc << "Grayscale";
       outputMess.append( " -type Grayscale " );
       }

    if (m_colorType->currentText() == i18n("Increase contrast"))
       {
       *proc << "+contrast";
       outputMess.append( " +contrast " );
       }

    if (m_colorType->currentText() == i18n("Monochrome"))
       {
       *proc << "-monochrome";
       outputMess.append( " -monochrome " );
       }

    if (m_colorType->currentText() == i18n("Negate"))
       {
       *proc << "-negate";
       outputMess.append( " -negate " );
       }

    if (m_colorType->currentText() == i18n("Normalize"))
       {
       *proc << "-normalize";
       outputMess.append( " -normalize " );
       }

    if (m_colorType->currentText() == i18n("Segment"))
       {
       *proc << "-segment";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_segmentCluster ) + "x";
       Temp2.append (Temp.setNum( m_segmentSmooth ));
       *proc << Temp2;
       outputMess.append( " -segment " + Temp2 );
       }

    if (m_colorType->currentText() == i18n("Trim"))
       {
       *proc << "-trim";
       outputMess.append( " -trim " );
       }

    return(outputMess);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotReadStd(KProcess* proc, char *buffer, int buflen)
{
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( m_listFile2Process_iterator->current() );
    item->changeOutputMess( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::ProcessDone(KProcess* proc)
{
    int ValRet = proc->exitStatus();
    qDebug ("Mogrify exit (%i)", ValRet);

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());

    switch (ValRet)
       {
       case 0:       // Process finished successfully !
          {
          item->changeResult(i18n("OK"));
          item->changeError(i18n("no processing error"));
          break;
          }
       case 15:      //  process aborted !
          {
          processAborted();
          break;
          }
       default :     // Processing error !
          {
          item->changeResult(i18n("Failed !!!"));
          item->changeError(i18n("cannot processing image file!"));
          break;
          }
       }

    ++*m_listFile2Process_iterator;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if ( m_listFile2Process_iterator->current() )
       startProcess();
    else
       {
       m_statusbar->setText(i18n("Process finished!"));
       endProcess();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("ColorImages Settings");
    m_config->writeEntry("ColorType", m_colorType->currentText());
    m_config->writeEntry("DepthValue", m_depthValue);
    m_config->writeEntry("FuzzDistance", m_fuzzDistance);
    m_config->writeEntry("SegmentCluster", m_segmentCluster);
    m_config->writeEntry("SegmentSmooth", m_segmentSmooth);
    m_config->writeEntry("SmallPreview", m_smallPreview->isChecked());

    m_config->sync();

    delete m_config;
    delete this;
}


////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void ColorImagesDialog::listImageFiles(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, QDir::Name);
    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileColorItem *item = new ImageFileColorItem(m_listFiles,
                                                              fi->fileName(),
                                                              ""
                                                              );
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotListDoubleClicked(QListViewItem *itemClicked)
{
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( itemClicked );

    if (m_processStatus == PROCESS_DONE)
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Image error processing"),
                                                   item->outputMess(),
                                                   i18n("Image \"%1\": %2\n\nThe ouput messages are:\n")
                                                        .arg(item->nameSrc()).arg(item->error())
                                                   );
       infoDialog->exec();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotProcessStop( void )
{
    // Try to kill the current process !
    if ( m_ProcessusProc->isRunning() == true ) m_ProcessusProc->kill(SIGTERM);

    // If kill operation failed, Stop the process at the next image !
    if ( m_processStatus == UNDER_PROCESS ) m_processStatus = STOP_PROCESS;

    processAborted();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotPreview(void)
{
    if ( m_listFiles->currentItem() == 0 )
       {
       KMessageBox::error(this, i18n("You must selected an item in the list for calculate the preview!"));
       return;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( m_listFiles->currentItem() );
    m_statusbar->setText(i18n("Preview under progress. Please wait..."));

    m_listFiles->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_labelColorType->setEnabled(false);
    m_colorType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_smallPreview->setEnabled(false);

    disconnect( this, SIGNAL(user2Clicked()),
                this, SLOT(slotProcessStart()));

    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotPreviewStop()));

    m_previewOutput ="convert";
    m_PreviewProc = new KProcess;
    *m_PreviewProc << "convert";

    if ( m_smallPreview->isChecked() )
       {
       *m_PreviewProc << "-crop" << "300x300+0+0";
       m_previewOutput.append( " -crop 300x300+0+0 ");
       }

    m_previewOutput.append(makeProcess(m_PreviewProc, album, item));

    *m_PreviewProc << album->getPath() + "/" + item->nameSrc();
    *m_PreviewProc << "-verbose";

    *m_PreviewProc << m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG";
    m_previewOutput.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + " " + m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG\n\n");

    connect(m_PreviewProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(PreviewProcessDone(KProcess*)));

    connect(m_PreviewProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    connect(m_PreviewProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    bool result = m_PreviewProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'convert' program from 'ImageMagick' package."
                                     "\nPlease, check your installation!"));
       endPreview();
       m_statusbar->setText("Preview aborted by user!");
       return;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotPreviewReadStd(KProcess* proc, char *buffer, int buflen)
{
    m_previewOutput.append( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::PreviewProcessDone(KProcess* proc)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( m_listFiles->currentItem() );
    int ValRet = proc->exitStatus();
    qDebug ("Convert exit (%i)", ValRet);

    if ( ValRet == 0 )
       {
       QString cropTitle = "";
       if ( m_smallPreview->isChecked() )
          cropTitle = i18n(" - small preview");

       ImagePreview *previewDialog = new ImagePreview(album->getPath() + "/" + item->nameSrc(),
                                         m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG",
                                         m_tmpFolder, m_smallPreview->isChecked(), false,
                                         m_colorType->currentText() + cropTitle);
       previewDialog->exec();

       KURL deletePreviewImage(m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG");
       KIO::NetAccess::del(deletePreviewImage);
       }
    else
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Preview error processing"),
                                                   m_previewOutput,
                                                   i18n("Cannot process preview for image \"%1\"."
                                                        "\nThe ouput messages are:\n")
                                                        .arg(item->nameSrc())
                                                   );
       infoDialog->exec();
       }

    endPreview();
    m_statusbar->setText("");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::slotPreviewStop( void )
{
    // Try to kill the current preview process !
    if ( m_PreviewProc->isRunning() == true ) m_PreviewProc->kill(SIGTERM);

    endPreview();
    m_statusbar->setText("Preview aborted by user!");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::endPreview(void)
{
    slotColorTypeChanged(m_colorType->currentText());
    m_listFiles->setEnabled(true);
    m_labelColorType->setEnabled(true);
    m_colorType->setEnabled(true);
    m_smallPreview->setEnabled(true);
    m_previewButton->setEnabled(true);
    showButtonCancel( true );

    setButtonText( User2, i18n("&Start") );

    disconnect(this, SIGNAL(user2Clicked()),
               this, SLOT(slotPreviewStop()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::endProcess(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumManager::instance()->refreshItemHandler(album->getTitle());
    m_processStatus = PROCESS_DONE;
    setButtonText( User2, i18n("&Close") );

    disconnect(this, SIGNAL(user2Clicked()),
               this, SLOT(slotProcessStop()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotOk()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ColorImagesDialog::processAborted(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileColorItem *item = static_cast<ImageFileColorItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());
    m_statusbar->setText(i18n("Process aborted by user!"));

    item->changeResult(i18n("Aborted !"));
    item->changeError(i18n("process aborted by user"));
    endProcess();
}
