//////////////////////////////////////////////////////////////////////////////
//
//    CONVERTIMAGESDIALOG.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qprocess.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpalette.h>

// Include files for KDE

#include <klocale.h>
#include <kprogress.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kimageio.h>
#include <kprocess.h>
#include <klineeditdlg.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/albuminfo.h>
#include <digikam/digikamio.h>

// Local includes

#include "convertimagesdialog.h"
#include "convertoptionsdialog.h"
#include "outputdialog.h"


class ImageFileConvertItem : public KListViewItem
{
public:
    ImageFileConvertItem(QListView * parent, QString const & nameSrc,
                         QString const & nameDest, QString const & result)
                       : KListViewItem( parent, nameSrc, nameDest, result),
                         _nameSrc(nameSrc), _nameDest(nameDest), _result(result)
    {}

    QString nameSrc()                   { return _nameSrc; }
    QString nameDest()                  { return _nameDest; }
    QString result()                    { return _result; }
    QString error()                     { return _error; }
    QString outputMess()                { return _outputMess; }

    void changeResult(QString text)     { setText(2, text); }
    void changeError(QString text)      { _error = text; }
    void changeNameDest(QString text)   { _nameDest = text; setText(1, _nameDest); }
    void changeOutputMess(QString text) { _outputMess.append(text); }

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(2) != i18n("OK") && text(2) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(2) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString _nameSrc;
    QString _nameDest;
    QString _result;
    QString _error;
    QString _outputMess;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

ConvertImagesDialog::ConvertImagesDialog(QWidget *parent, QStringList filesList)
                 : KDialogBase( parent, "ConvertImagesDialog", true,
                   i18n("Batch Convert Images options"), Help|User1|User2|Cancel,
                   Cancel, true, i18n("&About"), i18n("&Start"))
{
    m_convertStatus = NO_PROCESS;
    m_progressStatus = 0;
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_convertProc = 0L;

    setHelp("plugin-convertimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Convert images options"), box );

    m_labelImageFormat = new QLabel( i18n("Target Image files format:"), groupBox1);

    m_imagesFormat = new QComboBox(false, groupBox1);
    m_imagesFormat->insertItem("JPEG");
    m_imagesFormat->insertItem("PNG");
    m_imagesFormat->insertItem("TIFF");
    m_imagesFormat->insertItem("PPM");
    m_imagesFormat->insertItem("BMP");
    m_imagesFormat->insertItem("TGA");
    m_imagesFormat->setCurrentText("JPEG");
    whatsThis = i18n("<p>Select here the target image file format.<p>");
    whatsThis = whatsThis + i18n("<b>JPEG</b>: The Joint Photographic Experts Group file format is a "
                                 "good Web file format but it use a compression with data lost.<p>"
                                 "<b>PNG</b>: the Portable Network Graphics format is an extensible file "
                                 "format for the lossless, portable, well-compressed storage of raster images. "
                                 "PNG provides a patent-free replacement for GIF and can also replace many common "
                                 "uses of TIFF. PNG is designed to work well in online viewing applications, such "
                                 "as the World Wide Web, so it is fully streamable with a progressive display "
                                 "option. Also, PNG can store gamma and chromaticity data for improved color "
                                 "matching on heterogeneous platforms.");
    whatsThis = whatsThis + i18n("<p><b>TIFF</b>: the Tag Image File Format is a rather old standard that is "
                                 "still very popular today. It is a highly flexible and platform-independent "
                                 "format which is supported by numerous image processing applications and "
                                 "virtually all prepress software on the market.");
    whatsThis = whatsThis + i18n("<p><b>PPM</b>: the Portable Pixel Map file format is used as an "
                                 "intermediate format for storing color bitmap informations. PPM files "
                                 "may be either binary or ASCII and store pixel values up to 24 bits in size. "
                                 "This format generate very biggest sized text files for encode images without "
                                 "losing quality");
    whatsThis = whatsThis + i18n("<p><b>BMP</b>: the BitMaP file format is a popular image format from Win32 "
                                 "environment. It efficiently stores mapped or unmapped RGB graphics data with "
                                 "pixels 1-, 4-, 8-, or 24-bits in size. Data may be stored raw or compressed "
                                 "using a 4-bit or 8-bit RLE data compression algorithm. BMP is an excellent "
                                 "choice for a simple bitmap format which supports a wide range of RGB image "
                                 "data.");
    whatsThis = whatsThis + i18n("<p><b>TGA</b>: the TarGA image file format is one of the most widely used "
                                 "bitmap file formats for storage of 24 and 32 bits truecolor images.  "
                                 "TGA supports colormaps, alpha channel, gamma value, postage stamp image, "
                                 "textual information, and developer-definable data.");

    QWhatsThis::add( m_imagesFormat, whatsThis );
    m_labelImageFormat->setBuddy( m_imagesFormat );

    m_optionsButton = new QPushButton (groupBox1, "OptionButton");
    m_optionsButton->setText(i18n("Options"));
    QWhatsThis::add( m_optionsButton, i18n("<p>You can choose here the current image file format "
                                           "options using for the conversion process."));

    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox2 = new QGroupBox( 2, Qt::Horizontal, i18n("Files operations"), box );

    m_labelOverWrite = new QLabel (i18n("Overwrite mode:"), groupBox2);
    m_overWriteMode = new QComboBox( false, groupBox2 );
    m_overWriteMode->insertItem(i18n("Rename"));
    m_overWriteMode->insertItem(i18n("Ask"));
    m_overWriteMode->insertItem(i18n("Skip"));
    m_overWriteMode->insertItem(i18n("Always OverWrite"));
    m_overWriteMode->setCurrentText (i18n("Ask"));
    QWhatsThis::add( m_overWriteMode, i18n("<p>Select here the overwrite mode if your targets image "
                                           "files already exist.") );

    m_removeOriginal = new QCheckBox(i18n("Remove original"), groupBox2);
    QWhatsThis::add( m_removeOriginal, i18n("<p>If you enable this option, "
                                            "all originals images files will be removed.") );
    m_removeOriginal->setChecked( false );

    dvlay->addWidget( groupBox2 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 2, Qt::Horizontal, i18n("Target Album"), box );

    m_albumList = new QComboBox( false, groupBox3 );
    QStringList albumsList;

    for (Digikam::AlbumInfo *album=Digikam::AlbumManager::instance()->firstAlbum() ;
         album ; album = album->nextAlbum())
        {
        album->openDB();
        albumsList.append (album->getTitle());
        album->closeDB();
        }

    albumsList.sort();
    m_albumList->insertStringList(albumsList);
    m_albumList->setCurrentText (Digikam::AlbumManager::instance()->currentAlbum()->getTitle());
    QWhatsThis::add( m_albumList, i18n("<p>Select here the target Album for the renaming process.") );

    m_addNewAlbumButton = new QPushButton (groupBox3, "PushButton_AddNewAlbum");
    m_addNewAlbumButton->setText(i18n( "&Add new album") );
    m_addNewAlbumButton->setAutoRepeat( false );
    QWhatsThis::add( m_addNewAlbumButton, i18n( "Add a new Album in the Digikam Albums library."));

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    QGroupBox * groupBox4 = new QGroupBox( 1, Qt::Horizontal, i18n("Images to convert (%1 items)")
                                                              .arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox4 );
    m_listFiles->addColumn(i18n("Source image"));
    m_listFiles->addColumn(i18n("Target image"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(2);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 150 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here the operations results during the conversion. "
                                       "Double click on one item for more informations after the conversion "
                                       "process.") );

    m_statusbar = new QLabel( groupBox4, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>The current actived task..") );

    m_progress = new KProgress( groupBox4, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox4 );

    //---------------------------------------------

    connect(m_addNewAlbumButton, SIGNAL(clicked()),
            this, SLOT(slotAddNewAlbum()));

    connect(m_listFiles, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotListDoubleClicked(QListViewItem *)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));

    connect(m_optionsButton, SIGNAL(clicked()),
            this, SLOT(slotOptionsFileFormatClicked()));

    connect(m_imagesFormat, SIGNAL(activated(const QString &)),
            this, SLOT(slotImageFormatChanged(const QString &)));

    connect(m_overWriteMode, SIGNAL(activated(const QString &)),
            this, SLOT(slotOverWriteModeActived(const QString &)));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("ConvertImages Settings");

    m_imagesFormat->setCurrentText(m_config->readEntry("ImagesFormat", "JPEG"));
    if ( m_config->readEntry("CompressLossLess", "false") == "true")
       m_compressLossLess = true;
    else
       m_compressLossLess = false;

    m_JPEGPNGCompression = m_config->readNumEntry("JPEGPNGCompression", 75);
    m_TIFFCompressionAlgo = m_config->readEntry("TIFFCompressionAlgo", i18n("None"));
    m_TGACompressionAlgo = m_config->readEntry("TGACompressionAlgo", i18n("None"));

    m_overWriteMode->setCurrentItem(m_config->readNumEntry("OverWriteMode", 0));  // 'Ask' per default...

    if (m_config->readEntry("RemoveOriginal", "true") == "true")
        m_removeOriginal->setChecked( true );
    else
        m_removeOriginal->setChecked( false );

    delete m_config;
            
    //---------------------------------------------

    slotImageFormatChanged(m_imagesFormat->currentText());
    slotOverWriteModeActived(m_overWriteMode->currentText());
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

ConvertImagesDialog::~ConvertImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////


void ConvertImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch convert images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n\n"
                                  "This plugin use the \"convert\" program from \"ImageMagick\" package.\n"),
                                  i18n("About Digikam batch convert images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotOverWriteModeActived( const QString &string )
{
    if ( string == i18n("Always OverWrite") )
       {
       m_removeOriginal->setEnabled(false);
       m_removeOriginal->setChecked(false);
       }
    else
       m_removeOriginal->setEnabled(true);
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotAddNewAlbum( void )
{
    QDir libraryDir( Digikam::AlbumManager::instance()->getLibraryPath());

    if (!libraryDir.exists())
        {
        KMessageBox::error(0, i18n("Album Library has not been set correctly\n"
                                   "Please run Setup"));
        return;
        }

    bool ok;
    m_newDir = KLineEditDlg::getText(i18n("Enter New Album Name: "), "", &ok, this);
    if (!ok) return;

    KURL newAlbumURL(Digikam::AlbumManager::instance()->getLibraryPath());
    newAlbumURL.addPath(m_newDir);

    KIO::SimpleJob* job = KIO::mkdir(newAlbumURL);
    connect(job, SIGNAL(result(KIO::Job*)), this, SLOT(slot_onAlbumCreate(KIO::Job*)));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slot_onAlbumCreate(KIO::Job* job)
{
    if (job->error())
        {
        job->showErrorDialog(this);
        }
    else
        {
        m_albumList->insertStringList(m_newDir, -1);
        m_albumList->setCurrentText (m_newDir);
        }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotOptionsFileFormatClicked(void)
{
    QString Type = m_imagesFormat->currentText();
    ConvertOptionsDialog *optionsDialog = new ConvertOptionsDialog(0, Type);

    if (Type == "JPEG")
       {
       optionsDialog->m_JPEGPNGCompression->setValue(m_JPEGPNGCompression);
       optionsDialog->m_compressLossLess->setChecked(m_compressLossLess);
       }
    if (Type == "PNG")
       {
       optionsDialog->m_JPEGPNGCompression->setValue(m_JPEGPNGCompression);
       }
    if (Type == "TIFF")
       optionsDialog->m_TIFFCompressionAlgo->setCurrentText(m_TIFFCompressionAlgo);
    if (Type == "TGA")
       optionsDialog->m_TGACompressionAlgo->setCurrentText(m_TGACompressionAlgo);

    if ( optionsDialog->exec() == KMessageBox::Ok )
       {
       if (Type == "JPEG")
          {
          m_JPEGPNGCompression = optionsDialog->m_JPEGPNGCompression->value();
          m_compressLossLess = optionsDialog->m_compressLossLess->isChecked();
          }
       if (Type == "PNG")
          m_JPEGPNGCompression = optionsDialog->m_JPEGPNGCompression->value();
       if (Type == "TIFF")
          m_TIFFCompressionAlgo = optionsDialog->m_TIFFCompressionAlgo->currentText();
       if (Type == "TGA")
          m_TGACompressionAlgo = optionsDialog->m_TGACompressionAlgo->currentText();
       }

    delete optionsDialog;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotProcessStart( void )
{
    if ( m_removeOriginal->isChecked() == true )
        {
        if ( KMessageBox::warningContinueCancel(0,
             i18n("All original images files will be removed from the Album!\nDo you want to continue?"),
             i18n("Delete original images files"), KStdGuiItem::cont(),
             "Digikamplugin-ConvertImages-AlwaysRemomveOriginalFiles") != KMessageBox::Continue )
           return;
        }

    m_statusbar->setText(i18n("Process under progress. Please wait..."));
    m_convertStatus = UNDER_PROCESS;
    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotStop()));

    m_labelImageFormat->setEnabled(false);
    m_imagesFormat->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_labelOverWrite->setEnabled(false);
    m_overWriteMode->setEnabled(false);
    m_removeOriginal->setEnabled(false);
    m_addNewAlbumButton->setEnabled(false);
    m_albumList->setEnabled(false);

    m_listFile2Process_iterator = new QListViewItemIterator( m_listFiles );
    startProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool ConvertImagesDialog::startProcess(void)
{
    if ( m_convertStatus == STOP_PROCESS )
       {
       endProcess();
       return true;
       }

    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileConvertItem *item = static_cast<ImageFileConvertItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->setCurrentItem(item);

    KURL desturl(targetAlbum->getPath() + "/" + item->nameDest());

    if ( KIO::NetAccess::exists(desturl) == true )
       {
       switch (overwriteMode())
          {
          case OVERWRITE_ASK:
             {
             int ValRet = KMessageBox::warningYesNoCancel(0,
                          i18n("The destination file \"%1\" already exists!\n"
                          "Do you want overwrite it?").arg(item->nameDest()),
                          i18n("Overwrite destination image file"), KStdGuiItem::cont());

             if ( ValRet == KMessageBox::No )
                {
                item->changeResult(i18n("Skipped !"));
                item->changeError(i18n("destination image file already exists (skipped by user)!"));
                ++*m_listFile2Process_iterator;
                ++m_progressStatus;
                m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

                if ( m_listFile2Process_iterator->current() )
                   {
                   startProcess();
                   return true;
                   }
                else
                   {
                   endProcess();
                   return true;
                   }
                }

             if ( ValRet == KMessageBox::Cancel )
                {
                endProcess();
                return false;
                }

             break;
             }

          case OVERWRITE_RENAME:
             {
             QFileInfo *Target = new QFileInfo(targetAlbum->getPath() + "/" + item->nameDest());
             QString newFileName = RenameTargetImageFile(Target);

             if ( newFileName == QString::null )
                {
                item->changeResult(i18n("Failed !"));
                item->changeError(i18n("destination image file already exists and cannot be renamed!"));
                ++*m_listFile2Process_iterator;
                ++m_progressStatus;
                m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

                if ( m_listFile2Process_iterator->current() )
                   {
                   startProcess();
                   return true;
                   }
                else
                   {
                   endProcess();
                   return true;
                   }
                }
             else
                {
                QFileInfo *newTarget = new QFileInfo(newFileName);
                item->changeNameDest(newTarget->fileName());
                }

             break;
             }

          case OVERWRITE_SKIP:
             {
             item->changeResult(i18n("Skipped !"));
             item->changeError(i18n("destination image file already exists (skipped automaticly)!"));
             ++*m_listFile2Process_iterator;
             ++m_progressStatus;
             m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

             if ( m_listFile2Process_iterator->current() )
                {
                startProcess();
                return true;
                }
             else
                {
                endProcess();
                return true;
                }
             break;
             }

          case OVERWRITE_OVER:   // In this case do nothing : 'convert' default mode...
             break;

          default:
             {
             endProcess();
             return true;
             break;
             }
          }
       }

    QString CommandLine ="convert";
    m_convertProc = new KProcess;
    *m_convertProc << "convert";

    if (m_imagesFormat->currentText() == "JPEG")
       {
       if (m_compressLossLess == true)
          {
          *m_convertProc << "-compress" << "Lossless";
          CommandLine.append( " -compress Lossless ");
          }
       else
          {
          *m_convertProc << "-quality";
          QString Temp;
          *m_convertProc << Temp.setNum( m_JPEGPNGCompression );
          CommandLine.append( " -quality " + Temp.setNum( m_JPEGPNGCompression ) );
          }
       }

    if (m_imagesFormat->currentText() == "PNG")
       {
       *m_convertProc << "-quality";
       QString Temp;
       *m_convertProc << Temp.setNum( m_JPEGPNGCompression );
       CommandLine.append( " -quality " + Temp.setNum( m_JPEGPNGCompression ) );
       }

    if (m_imagesFormat->currentText() == "TIFF")
       {
       *m_convertProc << "-compress";
       if (m_TIFFCompressionAlgo == i18n("None"))
          {
          *m_convertProc << "None";
          CommandLine.append( " -compress None" );
          }
       else
          {
          *m_convertProc << m_TIFFCompressionAlgo;
          CommandLine.append( " -compress " + m_TIFFCompressionAlgo );
          }
       }

    if (m_imagesFormat->currentText() == "TGA")
       {
       *m_convertProc << "-compress";
       if (m_TGACompressionAlgo == i18n("None"))
          {
          *m_convertProc << "None";
          CommandLine.append( " -compress None" );
          }
       else
          {
          *m_convertProc << m_TGACompressionAlgo;
          CommandLine.append( " -compress " + m_TGACompressionAlgo );
          }
       }

    *m_convertProc << "-verbose";
    *m_convertProc << currentAlbum->getPath() + "/" + item->nameSrc();
    *m_convertProc << targetAlbum->getPath() + "/" + item->nameDest();
    CommandLine.append( " -verbose " + currentAlbum->getPath() + "/" + item->nameSrc() + " " +  targetAlbum->getPath() + "/" + item->nameDest() + "\n\n");

    item->changeOutputMess(CommandLine);

    connect(m_convertProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(ProcessDone(KProcess*)));

    connect(m_convertProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    connect(m_convertProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    bool result = m_convertProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'convert' program from 'ImageMagick' package.\n"
                          "Please, check your installation!"));
       return false;
       }

    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotReadStd(KProcess* proc, char *buffer, int buflen)
{
    ImageFileConvertItem *item = static_cast<ImageFileConvertItem*>( m_listFile2Process_iterator->current() );
    item->changeOutputMess( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::ProcessDone(KProcess* proc)
{
    int ValRet = proc->exitStatus();
    qDebug ("Convert exit (%i)", ValRet);

    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileConvertItem *item = static_cast<ImageFileConvertItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());

    switch (ValRet)
       {
       case 0:  // Process finished successfully !
          {
          item->changeResult(i18n("OK"));
          item->changeError(i18n("no conversion error"));

          // Save the comments for the converted image.

          currentAlbum->openDB();
          QString comments = currentAlbum->getItemComments(item->nameSrc());
          currentAlbum->closeDB();

          targetAlbum->openDB();
          targetAlbum->setItemComments(item->nameDest(), comments);
          targetAlbum->closeDB();

          if ( m_removeOriginal->isChecked() == true )
             {
             KURL deleteImage(currentAlbum->getPath() + "/" + item->nameSrc());
             if ( KIO::NetAccess::del(deleteImage) == false )
                {
                item->changeResult(i18n("Warning !"));
                item->changeError(i18n("cannot remove original image file!"));
                }
             }
          break;
          }
       case 15: //  process aborted !
          {
          processAborted();
          break;
          }
       default : // Processing error !
          {
          item->changeResult(i18n("Failed !!!"));
          item->changeError(i18n("cannot convert original image file!"));
          break;
          }
       }

    ++*m_listFile2Process_iterator;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if ( m_listFile2Process_iterator->current() )
       startProcess();
    else
       {
       m_statusbar->setText(i18n("Process finished!"));
       endProcess();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("ConvertImages Settings");
    m_config->writeEntry("ImagesFormat", m_imagesFormat->currentText());
    m_config->writeEntry("JPEGPNGCompression", m_JPEGPNGCompression);
    m_config->writeEntry("CompressLossLess", m_compressLossLess);
    m_config->writeEntry("TIFFCompressionAlgo", m_TIFFCompressionAlgo);
    m_config->writeEntry("TGACompressionAlgo", m_TGACompressionAlgo);
    m_config->writeEntry("OverWriteMode", m_overWriteMode->currentItem());
    m_config->writeEntry("RemoveOriginal", m_removeOriginal->isChecked());
    m_config->sync();

    delete m_config;
    delete this;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotImageFormatChanged(const QString &string)
{
    if ( string == "BMP" || string == "PPM" )
       m_optionsButton->setEnabled(false);
    else
       m_optionsButton->setEnabled(true);

    listImageFiles();
}

////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void ConvertImagesDialog::listImageFiles(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, QDir::Name);
    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileConvertItem *item = new ImageFileConvertItem(m_listFiles,
                                                                fi->fileName(),
                                                                oldFileName2NewFileName(fi),
                                                                ""
                                                                );
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotListDoubleClicked(QListViewItem *itemClicked)
{
    ImageFileConvertItem *item = static_cast<ImageFileConvertItem*>( itemClicked );

    if (m_convertStatus == PROCESS_DONE)
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Convert image error"),
                                                   item->outputMess(),
                                                   i18n("Image \"%1\": %2\n\nThe ouput messages are:\n")
                                                        .arg(item->nameSrc()).arg(item->error())
                                                   );
       infoDialog->exec();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::slotStop( void )
{
    // Try to kill the current process !
    if ( m_convertProc->isRunning() == true ) m_convertProc->kill(SIGTERM);

    // If kill operation failed, Stop the process at the next image !
    if ( m_convertStatus == UNDER_PROCESS ) m_convertStatus = STOP_PROCESS;

    processAborted();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString ConvertImagesDialog::oldFileName2NewFileName(QFileInfo *fi)
{
    QString Temp;

    Temp = fi->fileName().left( fi->fileName().findRev('.', -1) );      // The source file name without extension.
    Temp = Temp + "." + ImageFileExt(m_imagesFormat->currentText());    // Add new file extension.

    return Temp;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

int ConvertImagesDialog::overwriteMode(void)
{
    QString OverWrite = m_overWriteMode->currentText();

    if (OverWrite == i18n("Ask"))
        return OVERWRITE_ASK;

    if (OverWrite == i18n("Rename"))
        return OVERWRITE_RENAME;

    if (OverWrite == i18n("Skip"))
        return OVERWRITE_SKIP;

    if (OverWrite == i18n("Always OverWrite"))
        return OVERWRITE_OVER;

    return OVERWRITE_ASK;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::processAborted(void)
{
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    ImageFileConvertItem *item = static_cast<ImageFileConvertItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());
    m_statusbar->setText(i18n("Process aborted by user!"));
    
    item->changeResult(i18n("Aborted !"));
    item->changeError(i18n("process aborted by user"));
    KURL deleteImage(targetAlbum->getPath() + "/" + item->nameDest());
    KIO::NetAccess::del(deleteImage);                              // Try to delete de destination image !
    endProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void ConvertImagesDialog::endProcess(void)
{
    Digikam::AlbumInfo *currentAlbum = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumManager::instance()->refreshItemHandler(currentAlbum->getTitle());
    Digikam::AlbumInfo *targetAlbum = Digikam::AlbumManager::instance()->findAlbum( m_albumList->currentText() );
    Digikam::AlbumManager::instance()->refreshItemHandler(targetAlbum->getTitle());
    m_convertStatus = PROCESS_DONE;
    setButtonText( User2, i18n("&Close") );

    disconnect(this, SIGNAL(user2Clicked()), this, SLOT(slotStop()));

    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotOk()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString ConvertImagesDialog::RenameTargetImageFile(QFileInfo *fi)
{
    QString Temp;
    int Enumerator = 0;
    KURL NewDestUrl;

    do
       {
       ++Enumerator;
       Temp = Temp.setNum( Enumerator );
       NewDestUrl = fi->filePath().left( fi->filePath().findRev('.', -1)) + "_" + Temp + "." +
                                         ImageFileExt(m_imagesFormat->currentText());
       }
    while ( Enumerator < 100 && KIO::NetAccess::exists(NewDestUrl) == true );

    if (Enumerator == 100) return QString::null;

    return (NewDestUrl.path());
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString ConvertImagesDialog::ImageFileExt(QString Ext)
{
    if ( Ext == "TIFF" || Ext == "tiff" )
       return ("tif");
    else if ( Ext == "JPEG" || Ext == "jpeg" )
       return ("jpg");
    else
       return (Ext.lower());
}
