//////////////////////////////////////////////////////////////////////////////
//
//    EffectImagesDialog.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// C Ansi includes

extern "C"
{
#include <unistd.h>
}

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qprocess.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpalette.h>

// Include files for KDE

#include <kstandarddirs.h>
#include <klocale.h>
#include <kprogress.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kimageio.h>
#include <kprocess.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/digikamio.h>

// Local includes

#include "effectimagesdialog.h"
#include "effectoptionsdialog.h"
#include "outputdialog.h"
#include "imagepreview.h"


class ImageFileEffectItem : public KListViewItem
{
public:
    ImageFileEffectItem(QListView * parent, QString const & nameSrc, QString const & result)
      : KListViewItem( parent, nameSrc, result), _nameSrc(nameSrc), _result(result)
    {}

    QString nameSrc()                   { return _nameSrc; }
    QString result()                    { return _result; }
    QString error()                     { return _error; }
    QString outputMess()                { return _outputMess; }

    void changeResult(QString text)     { setText(1, text); }
    void changeError(QString text)      { _error = text; }
    void changeOutputMess(QString text) { _outputMess.append(text); }

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(1) != i18n("OK") && text(1) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(1) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString _nameSrc;
    QString _result;
    QString _error;
    QString _outputMess;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

EffectImagesDialog::EffectImagesDialog(QWidget *parent, QStringList filesList)
                 : KDialogBase( parent, "EffectImagesDialog", true,
                 i18n("Batch FX transforming Images options"), Help|User1|User2|Cancel,
                 Cancel, true, i18n("&About"), i18n("&Start"))
{
    // Init. Tmp folder
    KStandardDirs dir;
    m_tmpFolder = dir.saveLocation("tmp");

    m_processStatus = NO_PROCESS;
    m_progressStatus = 0;
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_ProcessusProc = 0L;
    m_PreviewProc = 0L;

    setHelp("plugin-effectimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Effect images options"), box );

    m_labelEffectType = new QLabel( i18n("FX type:"), groupBox1);

    m_EffectType = new QComboBox(false, groupBox1);
    m_EffectType->insertItem(i18n("Adaptative threshold"));
    m_EffectType->insertItem(i18n("Charcoal"));
    m_EffectType->insertItem(i18n("Detect edges"));
    m_EffectType->insertItem(i18n("Emboss"));
    m_EffectType->insertItem(i18n("Implode"));
    m_EffectType->insertItem(i18n("Paint"));
    m_EffectType->insertItem(i18n("Shade light"));
    m_EffectType->insertItem(i18n("Solarize"));
    m_EffectType->insertItem(i18n("Spread"));
    m_EffectType->insertItem(i18n("Swirl"));
    m_EffectType->insertItem(i18n("Wave"));
    m_EffectType->setCurrentText(i18n("Emboss"));
    whatsThis = i18n("<p>Select here the FX type for your images:<p>"
                     "<b>Adaptative threshold</b>: perform local adaptive thresholding. The algorithm "
                     "selects an individual threshold for each pixel based on the range of intensity "
                     "values in its local neighborhood. This allows for thresholding of an image whose "
                     "global intensity histogram doesn't contain distinctive peaks.<p>"
                     "<b>Charcoal</b>: simulate a charcoal drawing.<p>"
                     "<b>Detect edges</b>: detect edges within an image.<p>"
                     "<b>Emboss</b>: returns a grayscale image with a three-dimensional effect. The "
                     "algorithm convolve the image with a Gaussian operator of the given radius and "
                     "standard deviation.<p>"
                     "<b>Implode</b>: implode image pixels about the center.<p>"
                     "<b>Paint</b>: applies a special effect filter that simulates an oil painting.<p>"
                     "<b>Shade light</b>: shines a distant light on an image to create a three-dimensional "
                     "effect.<p>"
                     "<b>Solarize</b>: negate all pixels above the threshold level. This algorithm produces a "
                     "solarization effect seen when exposing a photographic film to light during the development "
                     "process.<p>"
                     "<b>Spread</b>: this is a special effects method that randomly displaces each pixel in a "
                     "block defined by the radius parameter.<p>"
                     "<b>Swirl</b>: swirls the pixels about the center of the image. <p>"
                     "<b>Wave</b>: creates a \"ripple\" effect in the image by shifting the pixels vertically "
                     "along a sine wave.<p>");

    QWhatsThis::add( m_EffectType, whatsThis );
    m_labelEffectType->setBuddy( m_EffectType );

    m_optionsButton = new QPushButton (groupBox1, "OptionButton");
    m_optionsButton->setText(i18n("&Options"));
    QWhatsThis::add( m_optionsButton, i18n("<p>You can choose here the current border type options using "
                                           "for the process."));

    m_previewButton = new QPushButton (groupBox1, "PreviewButton");
    m_previewButton->setText(i18n("&Preview"));
    QWhatsThis::add( m_previewButton, i18n("<p>You can build with this button an process preview for the "
                                           "current selected image in the list."));

    m_smallPreview = new QCheckBox(i18n("Small preview rendering"), groupBox1);
    QWhatsThis::add( m_smallPreview, i18n("<p>If you enable this option, "
                                          "all preview effects will be calculed on a small zone "
                                          "of the image (300x300 pixels on the top left corner). "
                                          "Enabled this option if you have a slow computer.") );
    m_smallPreview->setChecked( true );

    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 1, Qt::Horizontal,
                                           i18n("FX transforming images (%1 items)").arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox3 );
    m_listFiles->addColumn(i18n("Source image"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(1);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 250 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here the operations results during the filtering. "
                                       "Double click on one item for more informations after the process.") );

    m_statusbar = new QLabel( groupBox3, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>This is the current active task.") );

    m_progress = new KProgress( groupBox3, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    connect(m_listFiles, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotListDoubleClicked(QListViewItem *)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));

    connect(m_optionsButton, SIGNAL(clicked()),
            this, SLOT(slotOptionsClicked()));

    connect(m_previewButton, SIGNAL(clicked()),
            this, SLOT(slotPreview()));

    connect(m_EffectType, SIGNAL(activated(const QString &)),
            this, SLOT(slotEffectTypeChanged(const QString &)));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("EffectImages Settings");

    m_EffectType->setCurrentText(m_config->readEntry("EffectType", i18n("Normalize")));
    m_latWidth = m_config->readNumEntry("LatWidth", 50);
    m_latHeight = m_config->readNumEntry("LatHeight", 50);
    m_latOffset = m_config->readNumEntry("LatOffset", 1);
    m_charcoalRadius = m_config->readNumEntry("CharcoalRadius", 3);
    m_charcoalDeviation = m_config->readNumEntry("CharcoalDeviation", 3);
    m_edgeRadius = m_config->readNumEntry("EdgeRadius", 3);
    m_embossRadius = m_config->readNumEntry("EmbossRadius", 3);
    m_embossDeviation = m_config->readNumEntry("EmbossDeviation", 3);
    m_implodeFactor = m_config->readNumEntry("ImplodeFactor", 1);
    m_paintRadius = m_config->readNumEntry("PaintRadius", 3);
    m_shadeAzimuth = m_config->readNumEntry("ShadeAzimuth", 40);
    m_shadeElevation = m_config->readNumEntry("ShadeElevation", 40);
    m_solarizeFactor = m_config->readNumEntry("SolarizeFactor", 10);
    m_spreadRadius = m_config->readNumEntry("SpreadRadius", 3);
    m_swirlDegrees = m_config->readNumEntry("SwirlDegrees", 45);
    m_waveAmplitude = m_config->readNumEntry("WaveAmplitude", 50);
    m_waveLenght = m_config->readNumEntry("WaveLenght", 100);

    if (m_config->readEntry("SmallPreview", "true") == "true")
        m_smallPreview->setChecked( true );
    else
        m_smallPreview->setChecked( false );

    //---------------------------------------------

    listImageFiles();
    slotEffectTypeChanged(m_EffectType->currentText());

    delete m_config;
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

EffectImagesDialog::~EffectImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////

void EffectImagesDialog::closeEvent ( QCloseEvent *e )
{
    if (!e) return;

    if ( m_PreviewProc != 0L )
       if ( m_PreviewProc->isRunning() ) m_PreviewProc->kill(SIGTERM);

    if ( m_ProcessusProc != 0L )
       if ( m_ProcessusProc->isRunning() ) m_ProcessusProc->kill(SIGTERM);

    e->accept();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch FX transformations in images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n\n"
                                  "This plugin use the \"mogrify\" program from \"ImageMagick\" package.\n"),
                                  i18n("About Digikam batch FX transformations in images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotEffectTypeChanged(const QString &string)
{
    m_optionsButton->setEnabled(true);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotOptionsClicked(void)
{
    QString Type = m_EffectType->currentText();
    EffectOptionsDialog *optionsDialog = new EffectOptionsDialog(0, Type);

    if ( Type == i18n("Adaptative threshold") )
       {
       optionsDialog->m_latWidth->setValue(m_latWidth);
       optionsDialog->m_latHeight->setValue(m_latHeight);
       optionsDialog->m_latOffset->setValue(m_latOffset);
       }

    if ( Type == i18n("Charcoal") )
       {
       optionsDialog->m_charcoalRadius->setValue(m_charcoalRadius);
       optionsDialog->m_charcoalDeviation->setValue(m_charcoalDeviation);
       }

    if ( Type == i18n("Detect edges") )
       optionsDialog->m_edgeRadius->setValue(m_edgeRadius);

    if ( Type == i18n("Emboss") )
       {
       optionsDialog->m_embossRadius->setValue(m_embossRadius);
       optionsDialog->m_embossDeviation->setValue(m_embossDeviation);
       }

    if ( Type == i18n("Implode") )
       optionsDialog->m_implodeFactor->setValue(m_implodeFactor);

    if ( Type == i18n("Paint") )
       optionsDialog->m_paintRadius->setValue(m_paintRadius);

    if ( Type == i18n("Shade light") )
       {
       optionsDialog->m_shadeAzimuth->setValue(m_shadeAzimuth);
       optionsDialog->m_shadeElevation->setValue(m_shadeElevation);
       }

    if ( Type == i18n("Solarize") )
       optionsDialog->m_solarizeFactor->setValue(m_solarizeFactor);

    if ( Type == i18n("Spread") )
       optionsDialog->m_spreadRadius->setValue(m_spreadRadius);

    if ( Type == i18n("Swirl") )
       optionsDialog->m_swirlDegrees->setValue(m_swirlDegrees);

    if ( Type == i18n("Wave") )
       {
       optionsDialog->m_waveAmplitude->setValue(m_waveAmplitude);
       optionsDialog->m_waveLenght->setValue(m_waveLenght);
       }

    if ( optionsDialog->exec() == KMessageBox::Ok )
       {
       if ( Type == i18n("Adaptative threshold") )
          {
          m_latWidth = optionsDialog->m_latWidth->value();
          m_latHeight = optionsDialog->m_latHeight->value();
          m_latOffset = optionsDialog->m_latOffset->value();
          }

       if ( Type == i18n("Charcoal") )
          {
          m_charcoalRadius = optionsDialog->m_charcoalRadius->value();
          m_charcoalDeviation = optionsDialog->m_charcoalDeviation->value();
          }

       if ( Type == i18n("Detect edges") )
          m_edgeRadius = optionsDialog->m_edgeRadius->value();

       if ( Type == i18n("Emboss") )
          {
          m_embossRadius = optionsDialog->m_embossRadius->value();
          m_embossDeviation = optionsDialog->m_embossDeviation->value();
          }

       if ( Type == i18n("Implode") )
          m_implodeFactor = optionsDialog->m_implodeFactor->value();

       if ( Type == i18n("Paint") )
          m_paintRadius = optionsDialog->m_paintRadius->value();

       if ( Type == i18n("Shade light") )
          {
          m_shadeAzimuth = optionsDialog->m_shadeAzimuth->value();
          m_shadeElevation = optionsDialog->m_shadeElevation->value();
          }

       if ( Type == i18n("Solarize") )
          m_solarizeFactor = optionsDialog->m_solarizeFactor->value();

       if ( Type == i18n("Spread") )
          m_spreadRadius = optionsDialog->m_spreadRadius->value();

       if ( Type == i18n("Swirl") )
          m_swirlDegrees = optionsDialog->m_swirlDegrees->value();

       if ( Type == i18n("Wave") )
          {
          m_waveAmplitude = optionsDialog->m_waveAmplitude->value();
          m_waveLenght = optionsDialog->m_waveLenght->value();
          }
       }

    delete optionsDialog;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotProcessStart( void )
{
    m_processStatus = UNDER_PROCESS;
    m_statusbar->setText(i18n("Process under progress. Please wait..."));

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStop()));

    m_labelEffectType->setEnabled(false);
    m_EffectType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_smallPreview->setEnabled(false);

    m_listFile2Process_iterator = new QListViewItemIterator( m_listFiles );
    startProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool EffectImagesDialog::startProcess(void)
{
    if ( m_processStatus == STOP_PROCESS )
       {
       endProcess();
       return true;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->setCurrentItem(item);

    m_commandLine ="mogrify";
    m_ProcessusProc = new KProcess;
    *m_ProcessusProc << "mogrify";

    m_commandLine.append(makeProcess(m_ProcessusProc, album, item));

    *m_ProcessusProc << "-verbose";
    *m_ProcessusProc << album->getPath() + "/" + item->nameSrc();
    m_commandLine.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + "\n\n");

    item->changeOutputMess(m_commandLine);

    connect(m_ProcessusProc, SIGNAL(processExited(KProcess *)), this, SLOT(ProcessDone(KProcess*)));
    connect(m_ProcessusProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));
    connect(m_ProcessusProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    bool result = m_ProcessusProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'mogrify' program from 'ImageMagick' package."
                                     "\nPlease, check your installation!"));
       return false;
       }

    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString EffectImagesDialog::makeProcess(KProcess* proc, Digikam::AlbumInfo *album, ImageFileEffectItem *item)
{
    QString outputMess;
    KURL desturl(album->getPath() + "/" + item->nameSrc());

    if (m_EffectType->currentText() == i18n("Adaptative threshold"))
       {
       *proc << "-lat";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_latWidth ) + "x";
       Temp2.append ( Temp.setNum( m_latHeight ) + "+" );
       Temp2.append ( Temp.setNum( m_latOffset ) );
       *proc << Temp2;
       outputMess.append( " -lat " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Charcoal"))
       {
       *proc << "-charcoal";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_charcoalRadius ) + "x";
       Temp2.append ( Temp.setNum( m_charcoalDeviation ) );
       *proc << Temp2;
       outputMess.append( " -charcoal " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Detect edges"))
       {
       *proc << "-edge";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_edgeRadius );
       *proc << Temp2;
       outputMess.append( " -edge " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Emboss"))
       {
       *proc << "-emboss";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_embossRadius ) + "x";
       Temp2.append ( Temp.setNum( m_embossDeviation ) );
       *proc << Temp2;
       outputMess.append( " -emboss " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Implode"))
       {
       *proc << "-implode";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_implodeFactor );
       *proc << Temp2;
       outputMess.append( " -implode " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Paint"))
       {
       *proc << "-paint";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_paintRadius );
       *proc << Temp2;
       outputMess.append( " -paint " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Shade light"))
       {
       *proc << "-shade";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_shadeAzimuth ) + "x";
       Temp2.append ( Temp.setNum( m_shadeElevation ) );
       *proc << Temp2;
       outputMess.append( " -shade " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Solarize"))
       {
       *proc << "-solarize";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_solarizeFactor );
       *proc << Temp2;
       outputMess.append( " -solarize " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Spread"))
       {
       *proc << "-spread";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_spreadRadius );
       *proc << Temp2;
       outputMess.append( " -spread " + Temp2 );
       }

   if (m_EffectType->currentText() == i18n("Swirl"))
       {
       *proc << "-swirl";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_swirlDegrees );
       *proc << Temp2;
       outputMess.append( " -swirl " + Temp2 );
       }

    if (m_EffectType->currentText() == i18n("Wave"))
       {
       *proc << "-wave";
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_waveAmplitude ) + "x";
       Temp2.append ( Temp.setNum( m_waveLenght ) );
       *proc << Temp2;
       outputMess.append( " -wave " + Temp2 );
       }

    return(outputMess);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotReadStd(KProcess* proc, char *buffer, int buflen)
{
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( m_listFile2Process_iterator->current() );
    item->changeOutputMess( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::ProcessDone(KProcess* proc)
{
    int ValRet = proc->exitStatus();
    qDebug ("Mogrify exit (%i)", ValRet);

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());

    switch (ValRet)
       {
       case 0:       // Process finished successfully !
          {
          item->changeResult(i18n("OK"));
          item->changeError(i18n("no processing error"));
          break;
          }
       case 15:      //  process aborted !
          {
          processAborted();
          break;
          }
       default :     // Processing error !
          {
          item->changeResult(i18n("Failed !!!"));
          item->changeError(i18n("cannot processing image file!"));
          break;
          }
       }

    ++*m_listFile2Process_iterator;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if ( m_listFile2Process_iterator->current() )
       startProcess();
    else
       {
       m_statusbar->setText(i18n("Process finished!"));
       endProcess();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("EffectImages Settings");
    m_config->writeEntry("EffectType", m_EffectType->currentText());

    m_config->writeEntry("LatWidth", m_latWidth);
    m_config->writeEntry("LatHeight", m_latHeight);
    m_config->writeEntry("LatOffset", m_latOffset);
    m_config->writeEntry("CharcoalRadius", m_charcoalRadius);
    m_config->writeEntry("CharcoalDeviation", m_charcoalDeviation);
    m_config->writeEntry("EdgeRadius", m_edgeRadius);
    m_config->writeEntry("EmbossRadius", m_embossRadius);
    m_config->writeEntry("EmbossDeviation", m_embossDeviation);
    m_config->writeEntry("ImplodeFactor", m_implodeFactor);
    m_config->writeEntry("PaintRadius", m_paintRadius);
    m_config->writeEntry("ShadeAzimuth", m_shadeAzimuth);
    m_config->writeEntry("ShadeElevation", m_shadeElevation);
    m_config->writeEntry("SolarizeFactor", m_solarizeFactor);
    m_config->writeEntry("SpreadRadius", m_spreadRadius);
    m_config->writeEntry("SwirlDegrees", m_swirlDegrees);
    m_config->writeEntry("WaveAmplitude", m_waveAmplitude);
    m_config->writeEntry("WaveLenght", m_waveLenght);

    m_config->writeEntry("SmallPreview", m_smallPreview->isChecked());
    m_config->sync();

    delete m_config;
    delete this;
}


////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void EffectImagesDialog::listImageFiles(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, QDir::Name);
    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileEffectItem *item = new ImageFileEffectItem(m_listFiles,
                                                              fi->fileName(),
                                                              ""
                                                              );
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotListDoubleClicked(QListViewItem *itemClicked)
{
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( itemClicked );

    if (m_processStatus == PROCESS_DONE)
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Image error processing"),
                                                   item->outputMess(),
                                                   i18n("Image \"%1\": %2\n\n"
                                                        "The ouput messages are:\n")
                                                        .arg(item->nameSrc()).arg(item->error())
                                                   );
       infoDialog->exec();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotProcessStop( void )
{
    // Try to kill the current process !
    if ( m_ProcessusProc->isRunning() == true ) m_ProcessusProc->kill(SIGTERM);

    // If kill operation failed, Stop the process at the next image !
    if ( m_processStatus == UNDER_PROCESS ) m_processStatus = STOP_PROCESS;

    processAborted();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotPreview(void)
{
    if ( m_listFiles->currentItem() == 0 )
       {
       KMessageBox::error(this, i18n("You must selected an item in the list for calculate the preview!"));
       return;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( m_listFiles->currentItem() );
    m_statusbar->setText(i18n("Preview under progress. Please wait..."));

    m_listFiles->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_labelEffectType->setEnabled(false);
    m_EffectType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_smallPreview->setEnabled(false);

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotPreviewStop()));

    m_previewOutput ="convert";
    m_PreviewProc = new KProcess;
    *m_PreviewProc << "convert";

    if ( m_smallPreview->isChecked() )
       {
       *m_PreviewProc << "-crop" << "300x300+0+0";
       m_previewOutput.append( " -crop 300x300+0+0 ");
       }

    m_previewOutput.append(makeProcess(m_PreviewProc, album, item));

    *m_PreviewProc << album->getPath() + "/" + item->nameSrc();
    *m_PreviewProc << "-verbose";

    *m_PreviewProc << m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG";
    m_previewOutput.append( " -verbose " + album->getPath() + "/" + item->nameSrc()
                            + " " + m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG\n\n");

    connect(m_PreviewProc, SIGNAL(processExited(KProcess *)), this, SLOT(PreviewProcessDone(KProcess*)));
    connect(m_PreviewProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));
    connect(m_PreviewProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    bool result = m_PreviewProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'convert' program from 'ImageMagick' package.\n"
                                     "Please, check your installation!"));
       endPreview();
       m_statusbar->setText("Preview aborted by user!");
       return;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotPreviewReadStd(KProcess* proc, char *buffer, int buflen)
{
    m_previewOutput.append( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::PreviewProcessDone(KProcess* proc)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( m_listFiles->currentItem() );
    int ValRet = proc->exitStatus();
    qDebug ("Convert exit (%i)", ValRet);

    if ( ValRet == 0 )
       {
       QString cropTitle = "";
       if ( m_smallPreview->isChecked() )
          cropTitle = i18n(" - small preview");

       ImagePreview *previewDialog = new ImagePreview(album->getPath() + "/" + item->nameSrc(),
                                         m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG",
                                         m_tmpFolder, m_smallPreview->isChecked(), false,
                                         m_EffectType->currentText() + cropTitle);
       previewDialog->exec();

       KURL deletePreviewImage(m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG");
       KIO::NetAccess::del(deletePreviewImage);
       }
    else
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Preview error processing"),
                                                   m_previewOutput,
                                                   i18n("Cannot process preview for image \"%1\".\n"
                                                        "The ouput messages are:\n").arg(item->nameSrc())
                                                   );
       infoDialog->exec();
       }

    endPreview();
    m_statusbar->setText("");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::slotPreviewStop( void )
{
    // Try to kill the current preview process !

    if ( m_PreviewProc->isRunning() == true ) m_PreviewProc->kill(SIGTERM);

    endPreview();
    m_statusbar->setText("Preview aborted by user!");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::endPreview(void)
{
    slotEffectTypeChanged(m_EffectType->currentText());
    m_listFiles->setEnabled(true);
    m_labelEffectType->setEnabled(true);
    m_EffectType->setEnabled(true);
    m_smallPreview->setEnabled(true);
    m_previewButton->setEnabled(true);
    showButtonCancel( true );

    setButtonText( User2, i18n("&Start") );
    disconnect(this, SIGNAL(user2Clicked()), this, SLOT(slotPreviewStop()));
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::endProcess(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumManager::instance()->refreshItemHandler(album->getTitle());
    m_processStatus = PROCESS_DONE;
    setButtonText( User2, i18n("&Close") );
    disconnect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStop()));
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotOk()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void EffectImagesDialog::processAborted(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileEffectItem *item = static_cast<ImageFileEffectItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());
    m_statusbar->setText(i18n("Process aborted by user!"));

    item->changeResult(i18n("Aborted !"));
    item->changeError(i18n("process aborted by user"));
    endProcess();
}
