//////////////////////////////////////////////////////////////////////////////
//
//    FilterImagesDialog.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier dot gilles at free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

// C Ansi includes

extern "C"
{
#include <unistd.h>
}

// Include files for Qt

#include <qvbox.h>
#include <qlayout.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qprocess.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpalette.h>

// Include files for KDE

#include <kstandarddirs.h>
#include <klocale.h>
#include <kprogress.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <kinstance.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kdialogbase.h>
#include <klistview.h>
#include <kimageio.h>
#include <kprocess.h>
#include <kio/job.h>
#include <kio/jobclasses.h>
#include <kio/netaccess.h>
#include <kio/global.h>

// Digikam includes

#include <digikam/albummanager.h>
#include <digikam/digikamio.h>

// Local includes

#include "filterimagesdialog.h"
#include "filteroptionsdialog.h"
#include "outputdialog.h"
#include "imagepreview.h"


class ImageFileFilterItem : public KListViewItem
{
public:
    ImageFileFilterItem(QListView * parent, QString const & nameSrc, QString const & result)
      : KListViewItem( parent, nameSrc, result), _nameSrc(nameSrc), _result(result)
    {}

    QString nameSrc()                   { return _nameSrc; }
    QString result()                    { return _result; }
    QString error()                     { return _error; }
    QString outputMess()                { return _outputMess; }

    void changeResult(QString text)     { setText(1, text); }
    void changeError(QString text)      { _error = text; }
    void changeOutputMess(QString text) { _outputMess.append(text); }

    void paintCell (QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
    {
        QColorGroup _cg( cg );

        if (text(1) != i18n("OK") && text(1) != "" )
           {
           _cg.setColor( QColorGroup::Text, Qt::red );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }
        if (text(1) == i18n("OK") )
           {
           _cg.setColor( QColorGroup::Text, Qt::darkGreen );
           KListViewItem::paintCell( p, _cg, column, width, alignment );
           return;
           }

        KListViewItem::paintCell( p, cg, column, width, alignment );
    }

private:
    QString _nameSrc;
    QString _result;
    QString _error;
    QString _outputMess;
};


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

FilterImagesDialog::FilterImagesDialog(QWidget *parent, QStringList filesList)
                 : KDialogBase( parent, "FilterImagesDialog", true,
                 i18n("Batch Filtering Images options"), Help|User1|User2|Cancel, Cancel, true, i18n("&About"), i18n("&Start"))
{
    // Init. Tmp folder
    KStandardDirs dir;
    m_tmpFolder = dir.saveLocation("tmp");

    m_processStatus = NO_PROCESS;
    m_progressStatus = 0;
    m_selectedImageFiles = filesList;
    m_nbItem = m_selectedImageFiles.count();
    QString whatsThis;
    m_ProcessusProc = 0L;
    m_PreviewProc = 0L;

    setHelp("plugin-filterimages.anchor", "digikam");
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QGroupBox * groupBox1 = new QGroupBox( 2, Qt::Horizontal, i18n("Filter images options"), box );

    m_labelFilterType = new QLabel( i18n("Filter type:"), groupBox1);

    m_filterType = new QComboBox(false, groupBox1);
    m_filterType->insertItem(i18n("Add noise"));
    m_filterType->insertItem(i18n("Antialias"));
    m_filterType->insertItem(i18n("Blur"));
    m_filterType->insertItem(i18n("Despeckle"));
    m_filterType->insertItem(i18n("Enhance"));
    m_filterType->insertItem(i18n("Median"));
    m_filterType->insertItem(i18n("Noise reduction"));
    m_filterType->insertItem(i18n("Sharpen"));
    m_filterType->insertItem(i18n("Unsharp"));
    m_filterType->setCurrentText(i18n("Sharpen"));
    whatsThis = i18n("<p>Select here the filter type for your images:<p>"
                     "<b>Add noise</b>: add noise in an image.<p>"
                     "<b>Antialias</b>: remove pixel aliasing.<p>"
                     "<b>Blur</b>: blur the image with a Gaussian operator.<p>"
                     "<b>Despeckle</b>: reduces the speckle noise in an image while perserving the "
                     "edges of the original image.<p>"
                     "<b>Enhance</b>: apply a digital filter to enhance a noisy image.<p>"
                     "<b>Median</b>: apply a median filter to an image.<p>"
                     "<b>Noise reduction</b>: reduce noise in an image. <p>"
                     "<b>Sharpen</b>: sharpen the image with a Gaussian operator.<p>"
                     "<b>Unsharp</b>: sharpen the image with an unsharp mask operator.<p>");

    QWhatsThis::add( m_filterType, whatsThis );
    m_labelFilterType->setBuddy( m_filterType );

    m_optionsButton = new QPushButton (groupBox1, "OptionButton");
    m_optionsButton->setText(i18n("&Options"));
    QWhatsThis::add( m_optionsButton, i18n("<p>You can choose here the current border type "
                                           "options using for the process."));

    m_previewButton = new QPushButton (groupBox1, "PreviewButton");
    m_previewButton->setText(i18n("&Preview"));
    QWhatsThis::add( m_previewButton, i18n("<p>You can build with this button an process preview "
                                           "for the current selected image in the list."));

    m_smallPreview = new QCheckBox(i18n("Small preview rendering"), groupBox1);
    QWhatsThis::add( m_smallPreview, i18n("<p>If you enable this option, "
                                          "all preview effects will be calculed on a small zone "
                                          "of the image (300x300 pixels on the top left corner). "
                                          "Enabled this option if you have a slow computer.") );
    m_smallPreview->setChecked( true );

    dvlay->addWidget( groupBox1 );

    //---------------------------------------------

    QGroupBox * groupBox3 = new QGroupBox( 1, Qt::Horizontal, i18n("Filtering images (%1 items)").arg(m_nbItem), box );

    m_listFiles = new KListView( groupBox3 );
    m_listFiles->addColumn(i18n("Source image"));
    m_listFiles->addColumn(i18n("Result"));
    m_listFiles->setSorting(1);
    m_listFiles->setItemMargin(3);
    m_listFiles->setMaximumHeight( 250 );
    m_listFiles->setResizeMode(QListView::LastColumn);
    m_listFiles->setSelectionMode(QListView::Single);
    m_listFiles->setAllColumnsShowFocus ( true );
    QWhatsThis::add( m_listFiles, i18n("<p>You can see here the operations results during the "
                                       "filtering. Double click on one item for more informations after "
                                       "the process.") );

    m_statusbar = new QLabel( groupBox3, "ProcessMessagesFrame" );
    m_statusbar->setMinimumHeight( 25 );
    m_statusbar->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_statusbar->setAlignment(AlignCenter|WordBreak|ExpandTabs);
    QWhatsThis::add( m_statusbar, i18n("<p>This is the current active task.") );

    m_progress = new KProgress( groupBox3, "Progress" );
    m_progress->setMinimumHeight( 20 );
    m_progress->setRange(0, 100);
    m_progress->setValue(0);

    dvlay->addWidget( groupBox3 );

    //---------------------------------------------

    connect(m_listFiles, SIGNAL(doubleClicked(QListViewItem *)),
            this, SLOT(slotListDoubleClicked(QListViewItem *)));

    connect(this, SIGNAL(user1Clicked()),
            this, SLOT(slotAbout()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));

    connect(m_optionsButton, SIGNAL(clicked()),
            this, SLOT(slotOptionsClicked()));

    connect(m_previewButton, SIGNAL(clicked()),
            this, SLOT(slotPreview()));

    connect(m_filterType, SIGNAL(activated(const QString &)),
            this, SLOT(slotFilterTypeChanged(const QString &)));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("FilterImages Settings");

    m_filterType->setCurrentText(m_config->readEntry("FilterType", i18n("Sharpen")));
    m_noiseType = m_config->readEntry("NoiseType", i18n("Gaussian"));
    m_blurRadius = m_config->readNumEntry("BlurRadius", 3);
    m_blurDeviation = m_config->readNumEntry("BlurDeviation", 1);
    m_medianRadius = m_config->readNumEntry("MedianRadius", 3);
    m_noiseRadius = m_config->readNumEntry("NoiseRadius", 3);
    m_sharpenRadius = m_config->readNumEntry("SharpenRadius", 3);
    m_sharpenDeviation = m_config->readNumEntry("SharpenDeviation", 1);
    m_unsharpenRadius = m_config->readNumEntry("UnsharpenRadius", 3);
    m_unsharpenDeviation = m_config->readNumEntry("UnsharpenDeviation", 1);
    m_unsharpenPercent = m_config->readNumEntry("UnsharpenPercent", 3);
    m_unsharpenThreshold = m_config->readNumEntry("UnsharpenThreshold", 1);

    if (m_config->readEntry("SmallPreview", "true") == "true")
        m_smallPreview->setChecked( true );
    else
        m_smallPreview->setChecked( false );

    //---------------------------------------------

    listImageFiles();
    slotFilterTypeChanged(m_filterType->currentText());

    delete m_config;
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

FilterImagesDialog::~FilterImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////

void FilterImagesDialog::closeEvent ( QCloseEvent *e )
{
    if (!e) return;

    if ( m_PreviewProc != 0L )
       if ( m_PreviewProc->isRunning() ) m_PreviewProc->kill(SIGTERM);

    if ( m_ProcessusProc != 0L )
       if ( m_ProcessusProc->isRunning() ) m_ProcessusProc->kill(SIGTERM);

    e->accept();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for batch filtering images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier dot gilles at free.fr\n\n"
                                  "This plugin use the \"mogrify\" program from \"ImageMagick\" package.\n"),
                                  i18n("About Digikam batch filtering images"));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotFilterTypeChanged(const QString &string)
{
    if ( string == i18n("Antialias") || string == i18n("Despeckle") || string == i18n("Enhance") )
       m_optionsButton->setEnabled(false);
    else
       m_optionsButton->setEnabled(true);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotOptionsClicked(void)
{
    QString Type = m_filterType->currentText();
    FilterOptionsDialog *optionsDialog = new FilterOptionsDialog(0, Type);

    if ( Type == i18n("Add noise") )
       optionsDialog->m_noiseType->setCurrentText(m_noiseType);

    if ( Type == i18n("Blur") )
       {
       optionsDialog->m_blurRadius->setValue(m_blurRadius);
       optionsDialog->m_blurDeviation->setValue(m_blurDeviation);
       }

    if ( Type == i18n("Median") )
       optionsDialog->m_medianRadius->setValue(m_medianRadius);

    if ( Type == i18n("Noise reduction") )
       optionsDialog->m_noiseRadius->setValue(m_noiseRadius);

    if ( Type == i18n("Sharpen") )
       {
       optionsDialog->m_sharpenRadius->setValue(m_sharpenRadius);
       optionsDialog->m_sharpenDeviation->setValue(m_sharpenDeviation);
       }

    if ( Type == i18n("Unsharp") )
       {
       optionsDialog->m_unsharpenRadius->setValue(m_unsharpenRadius);
       optionsDialog->m_unsharpenDeviation->setValue(m_unsharpenDeviation);
       optionsDialog->m_unsharpenPercent->setValue(m_unsharpenPercent);
       optionsDialog->m_unsharpenThreshold->setValue(m_unsharpenThreshold);
       }

    if ( optionsDialog->exec() == KMessageBox::Ok )
       {
       if ( Type == i18n("Add noise") )
          m_noiseType = optionsDialog->m_noiseType->currentText();

       if ( Type == i18n("Blur") )
          {
          m_blurRadius = optionsDialog->m_blurRadius->value();
          m_blurDeviation = optionsDialog->m_blurDeviation->value();
          }

       if ( Type == i18n("Median") )
          m_medianRadius = optionsDialog->m_medianRadius->value();

       if ( Type == i18n("Noise reduction") )
          m_noiseRadius = optionsDialog->m_noiseRadius->value();

       if ( Type == i18n("Sharpen") )
          {
          m_sharpenRadius = optionsDialog->m_sharpenRadius->value();
          m_sharpenDeviation = optionsDialog->m_sharpenDeviation->value();
          }

       if ( Type == i18n("Unsharp") )
          {
          m_unsharpenRadius = optionsDialog->m_unsharpenRadius->value();
          m_unsharpenDeviation = optionsDialog->m_unsharpenDeviation->value();
          m_unsharpenPercent = optionsDialog->m_unsharpenPercent->value();
          m_unsharpenThreshold = optionsDialog->m_unsharpenThreshold->value();
          }
       }

    delete optionsDialog;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotProcessStart( void )
{
    m_processStatus = UNDER_PROCESS;
    m_statusbar->setText(i18n("Process under progress. Please wait..."));

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStop()));

    m_labelFilterType->setEnabled(false);
    m_filterType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_smallPreview->setEnabled(false);

    m_listFile2Process_iterator = new QListViewItemIterator( m_listFiles );
    startProcess();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilterImagesDialog::startProcess(void)
{
    if ( m_processStatus == STOP_PROCESS )
       {
       endProcess();
       return true;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->setCurrentItem(item);

    m_commandLine ="mogrify";
    m_ProcessusProc = new KProcess;
    *m_ProcessusProc << "mogrify";

    m_commandLine.append(makeProcess(m_ProcessusProc, album, item));

    *m_ProcessusProc << "-verbose";
    *m_ProcessusProc << album->getPath() + "/" + item->nameSrc();
    m_commandLine.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + "\n\n");

    item->changeOutputMess(m_commandLine);

    connect(m_ProcessusProc, SIGNAL(processExited(KProcess *)), this, SLOT(ProcessDone(KProcess*)));
    connect(m_ProcessusProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));
    connect(m_ProcessusProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotReadStd(KProcess*, char*, int)));

    bool result = m_ProcessusProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'mogrify' program from 'ImageMagick' package.\n"
                                     "Please, check your installation!"));
       return false;
       }

    return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString FilterImagesDialog::makeProcess(KProcess* proc, Digikam::AlbumInfo *album, ImageFileFilterItem *item)
{
    QString outputMess;
    KURL desturl(album->getPath() + "/" + item->nameSrc());

    if (m_filterType->currentText() == i18n("Add noise"))
       {
       QString Temp;
       if ( m_noiseType == i18n("Uniform") ) Temp = "Uniform";
       if ( m_noiseType == i18n("Gaussian") ) Temp = "Gaussian";
       if ( m_noiseType == i18n("Multiplicative") ) Temp = "Multiplicative";
       if ( m_noiseType == i18n("Impulse") ) Temp = "Impulse";
       if ( m_noiseType == i18n("Laplacian") ) Temp = "Laplacian";
       if ( m_noiseType == i18n("Poisson") ) Temp = "Poisson";
       *proc << "+noise" << Temp;
       outputMess.append( " +noise " + Temp );
       }

    if (m_filterType->currentText() == i18n("Antialias"))
       {
       *proc << "-antialias";
       outputMess.append( " -antialias " );
       }

    if (m_filterType->currentText() == i18n("Blur"))
       {
       *proc << "-blur";
       outputMess.append( " -blur " );
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_blurRadius ) + "x";
       Temp2.append(Temp.setNum( m_blurDeviation ));
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    if (m_filterType->currentText() == i18n("Despeckle"))
       {
       *proc << "-despeckle";
       outputMess.append( " -despeckle " );
       }

    if (m_filterType->currentText() == i18n("Enhance"))
       {
       *proc << "-enhance";
       outputMess.append( " -enhance " );
       }

    if (m_filterType->currentText() == i18n("Median"))
       {
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_medianRadius );
       *proc << "-median" << Temp2;
       outputMess.append( " -median " + Temp2 );
       }

    if (m_filterType->currentText() == i18n("Noise reduction"))
       {
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_noiseRadius );
       *proc << "-noise" << Temp2;
       outputMess.append( " -noise " + Temp2 );
       }

    if (m_filterType->currentText() == i18n("Sharpen"))
       {
       *proc << "-sharpen";
       outputMess.append( " -sharpen " );
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_sharpenRadius ) + "x";
       Temp2.append(Temp.setNum( m_sharpenDeviation ));
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    if (m_filterType->currentText() == i18n("Unsharp"))
       {
       *proc << "-unsharp";
       outputMess.append( " -unsharp " );
       QString Temp, Temp2;
       Temp2 = Temp.setNum( m_unsharpenRadius ) + "x";
       Temp2.append(Temp.setNum( m_unsharpenDeviation ));
       Temp2.append( "+" + Temp.setNum( m_unsharpenPercent ));
       Temp2.append( "+" + Temp.setNum( m_unsharpenThreshold ));
       *proc << Temp2;
       outputMess.append( Temp2 );
       }

    return(outputMess);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotReadStd(KProcess* proc, char *buffer, int buflen)
{
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( m_listFile2Process_iterator->current() );
    item->changeOutputMess( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::ProcessDone(KProcess* proc)
{
    int ValRet = proc->exitStatus();
    qDebug ("Mogrify exit (%i)", ValRet);

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());

    switch (ValRet)
       {
       case 0:       // Process finished successfully !
          {
          item->changeResult(i18n("OK"));
          item->changeError(i18n("no processing error"));
          break;
          }
       case 15:      //  process aborted !
          {
          processAborted();
          break;
          }
       default :     // Processing error !
          {
          item->changeResult(i18n("Failed !!!"));
          item->changeError(i18n("cannot processing image file!"));
          break;
          }
       }

    ++*m_listFile2Process_iterator;
    ++m_progressStatus;
    m_progress->setValue((int)((float)m_progressStatus *(float)100 / (float)m_nbItem));

    if ( m_listFile2Process_iterator->current() )
       startProcess();
    else
       {
       m_statusbar->setText(i18n("Process finished!"));
       endProcess();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("FilterImages Settings");
    m_config->writeEntry("FilterType", m_filterType->currentText());
    m_config->writeEntry("NoiseType", m_noiseType);
    m_config->writeEntry("BlurRadius", m_blurRadius);
    m_config->writeEntry("BlurDeviation", m_blurDeviation);
    m_config->writeEntry("MedianRadius", m_medianRadius);
    m_config->writeEntry("NoiseRadius", m_noiseRadius);
    m_config->writeEntry("SharpenRadius", m_sharpenRadius);
    m_config->writeEntry("SharpenDeviation", m_sharpenDeviation);
    m_config->writeEntry("UnsharpenRadius", m_unsharpenRadius);
    m_config->writeEntry("UnsharpenDeviation", m_unsharpenDeviation);
    m_config->writeEntry("UnsharpenPercent", m_unsharpenPercent);
    m_config->writeEntry("UnsharpenThreshold", m_unsharpenThreshold);
    m_config->writeEntry("SmallPreview", m_smallPreview->isChecked());

    m_config->sync();

    delete m_config;
    delete this;
}


////////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////

void FilterImagesDialog::listImageFiles(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    QDir *dir = new QDir(album->getPath());
    const QFileInfoList* fileinfolist;
    m_listFiles->clear();

    fileinfolist = dir->entryInfoList(QDir::Files | QDir::NoSymLinks, QDir::Name);
    QFileInfoListIterator it(*fileinfolist);
    QFileInfo* fi;

    while( (fi = it.current() ) )
       {
       if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

       if ( m_selectedImageFiles.find(fi->filePath()) != m_selectedImageFiles.end() )
          {
          ImageFileFilterItem *item = new ImageFileFilterItem(m_listFiles,
                                                              fi->fileName(),
                                                              ""
                                                              );
          }

       ++it;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotListDoubleClicked(QListViewItem *itemClicked)
{
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( itemClicked );

    if (m_processStatus == PROCESS_DONE)
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Image error processing"),
                                                   item->outputMess(),
                                                   i18n("Image \"%1\": %2\n\nThe ouput messages are:\n")
                                                        .arg(item->nameSrc()).arg(item->error())
                                                   );
       infoDialog->exec();
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotProcessStop( void )
{
    // Try to kill the current process !
    if ( m_ProcessusProc->isRunning() == true ) m_ProcessusProc->kill(SIGTERM);

    // If kill operation failed, Stop the process at the next image !
    if ( m_processStatus == UNDER_PROCESS ) m_processStatus = STOP_PROCESS;

    processAborted();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotPreview(void)
{
    if ( m_listFiles->currentItem() == 0 )
       {
       KMessageBox::error(this, i18n("You must selected an item in the list for calculate the preview!"));
       return;
       }

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( m_listFiles->currentItem() );
    m_statusbar->setText(i18n("Preview under progress. Please wait..."));

    m_listFiles->setEnabled(false);
    m_previewButton->setEnabled(false);
    m_labelFilterType->setEnabled(false);
    m_filterType->setEnabled(false);
    m_optionsButton->setEnabled(false);
    m_smallPreview->setEnabled(false);

    disconnect( this, SIGNAL(user2Clicked()), this, SLOT(slotProcessStart()));
    showButtonCancel( false );
    setButtonText( User2, i18n("&Stop") );
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotPreviewStop()));

    m_previewOutput ="convert";
    m_PreviewProc = new KProcess;
    *m_PreviewProc << "convert";

    if ( m_smallPreview->isChecked() )
       {
       *m_PreviewProc << "-crop" << "300x300+0+0";
       m_previewOutput.append( " -crop 300x300+0+0 ");
       }

    m_previewOutput.append(makeProcess(m_PreviewProc, album, item));

    *m_PreviewProc << album->getPath() + "/" + item->nameSrc();
    *m_PreviewProc << "-verbose";

    *m_PreviewProc << m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG";
    m_previewOutput.append( " -verbose " + album->getPath() + "/" + item->nameSrc() + " " + m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG\n\n");

    connect(m_PreviewProc, SIGNAL(processExited(KProcess *)),
            this, SLOT(PreviewProcessDone(KProcess*)));

    connect(m_PreviewProc, SIGNAL(receivedStdout(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    connect(m_PreviewProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
            this, SLOT(slotPreviewReadStd(KProcess*, char*, int)));

    bool result = m_PreviewProc->start(KProcess::NotifyOnExit, KProcess::All);

    if(!result)
       {
       KMessageBox::error(this, i18n("Cannot start 'convert' program from 'ImageMagick' package.\n"
                                     "Please, check your installation!"));
       endPreview();
       m_statusbar->setText("Preview aborted by user!");
       return;
       }
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotPreviewReadStd(KProcess* proc, char *buffer, int buflen)
{
    m_previewOutput.append( QString::fromLocal8Bit(buffer, buflen) );
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::PreviewProcessDone(KProcess* proc)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( m_listFiles->currentItem() );
    int ValRet = proc->exitStatus();
    qDebug ("Convert exit (%i)", ValRet);

    if ( ValRet == 0 )
       {
       QString cropTitle = "";
       if ( m_smallPreview->isChecked() )
          cropTitle = i18n(" - small preview");

       ImagePreview *previewDialog = new ImagePreview(album->getPath() + "/" + item->nameSrc(),
                                         m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG",
                                         m_tmpFolder, m_smallPreview->isChecked(), false,
                                         m_filterType->currentText() + cropTitle);
       previewDialog->exec();

       KURL deletePreviewImage(m_tmpFolder + "/" + QString::number(getpid()) + "preview.PNG");
       KIO::NetAccess::del(deletePreviewImage);
       }
    else
       {
       OutputDialog *infoDialog = new OutputDialog(0,
                                                   i18n("Preview error processing"),
                                                   m_previewOutput,
                                                   i18n("Cannot process preview for image \"%1\".\n"
                                                   "The ouput messages are:\n").arg(item->nameSrc())
                                                   );
       infoDialog->exec();
       }

    endPreview();
    m_statusbar->setText("");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::slotPreviewStop( void )
{
    // Try to kill the current preview process !
    if ( m_PreviewProc->isRunning() == true ) m_PreviewProc->kill(SIGTERM);

    endPreview();
    m_statusbar->setText("Preview aborted by user!");
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::endPreview(void)
{
    slotFilterTypeChanged(m_filterType->currentText());
    m_listFiles->setEnabled(true);
    m_labelFilterType->setEnabled(true);
    m_filterType->setEnabled(true);
    m_smallPreview->setEnabled(true);
    m_previewButton->setEnabled(true);
    showButtonCancel( true );

    setButtonText( User2, i18n("&Start") );

    disconnect(this, SIGNAL(user2Clicked()),
               this, SLOT(slotPreviewStop()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotProcessStart()));
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::endProcess(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    Digikam::AlbumManager::instance()->refreshItemHandler(album->getTitle());
    m_processStatus = PROCESS_DONE;
    setButtonText( User2, i18n("&Close") );

    disconnect(this, SIGNAL(user2Clicked()),
               this, SLOT(slotProcessStop()));

    connect(this, SIGNAL(user2Clicked()),
            this, SLOT(slotOk()));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void FilterImagesDialog::processAborted(void)
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    ImageFileFilterItem *item = static_cast<ImageFileFilterItem*>( m_listFile2Process_iterator->current() );
    m_listFiles->ensureItemVisible(m_listFiles->currentItem());
    m_statusbar->setText(i18n("Process aborted by user!"));

    item->changeResult(i18n("Aborted !"));
    item->changeError(i18n("process aborted by user"));
    endProcess();
}
